@@ -1,17 +1,18 @@
+# pylint: disable=unused-import,redefined-builtin,undefined-variable
 from __future__ import absolute_import, division, unicode_literals
 import os
 import sys
 try:
-    import urllib2 as parse
+    import urllib2 as parse  # noqa
 except ImportError:
     # Python 3
-    from urllib import parse
+    from urllib import parse  # noqa
 
 try:
     # Python 2.7+
-    from collections import OrderedDict as odict
+    from collections import OrderedDict as odict  # noqa
 except ImportError:
-    from .ordered_dict import OrderedDict as odict
+    from .ordered_dict import OrderedDict as odict  # noqa
 
 
 PY2 = sys.version_info[0] == 2
@@ -23,11 +24,9 @@ ENCODING = 'utf-8'
 
 if PY3:
     def bstr(x, encoding=ENCODING):
-        # pylint: disable=bytes-builtin
         return bytes(x, encoding=encoding)
 
 elif PY26_PLUS:
-    # pylint: disable=bytes-builtin
     bstr = bytes
 else:
     # Python <= 2.5
@@ -40,15 +39,16 @@ if PY3:
     int_types = (int,)
     maxsize = sys.maxsize
     ustr = str
-    unichr = chr
+    uchr = chr
 else:
     bchr = chr
     maxsize = 2 ** 31
     # pylint: disable=unicode-builtin
-    ustr = unicode
+    ustr = unicode  # noqa
     # pylint: disable=unichr-builtin
-    unichr = unichr
-    int_types = (int, long)  # pylint: disable=long-builtin
+    uchr = unichr  # noqa
+    # pylint: disable=long-builtin
+    int_types = (int, long)  # noqa
 
 
 def setenv(key, value):
@@ -57,7 +57,7 @@ def setenv(key, value):
     Why?  win32 requires putenv().  UNIX only requires os.environ.
 
     """
-    if not PY3 and type(value) is ustr:
+    if not PY3 and isinstance(value, ustr):
         value = value.encode(ENCODING, 'replace')
     os.environ[key] = value
     os.putenv(key, value)
@@ -67,7 +67,7 @@ def unsetenv(key):
     """Compatibility wrapper for unsetting environment variables"""
     try:
         del os.environ[key]
-    except:
+    except KeyError:
         pass
     if hasattr(os, 'unsetenv'):
         os.unsetenv(key)
