@@ -26,19 +26,17 @@ class _DiffHunk(object):
         return self.first_line_idx + len(self.lines) - 1
 
 
-def _parse_range_str(range_str):
+def parse_range_str(range_str):
     if ',' in range_str:
         begin, end = range_str.split(',', 1)
         return int(begin), int(end)
-    else:
-        return int(range_str), 1
+    return int(range_str), 1
 
 
 def _format_range(start, count):
     if count == 1:
         return str(start)
-    else:
-        return '%d,%d' % (start, count)
+    return '%d,%d' % (start, count)
 
 
 def _format_hunk_header(old_start, old_count, new_start, new_count,
@@ -53,17 +51,13 @@ def _parse_diff(diff_text):
     for line_idx, line in enumerate(diff_text.split('\n')):
         match = _HUNK_HEADER_RE.match(line)
         if match:
-            old_start, old_count = _parse_range_str(match.group(1))
-            new_start, new_count = _parse_range_str(match.group(2))
+            old_start, old_count = parse_range_str(match.group(1))
+            new_start, new_count = parse_range_str(match.group(2))
             heading = match.group(3)
             hunks.append(_DiffHunk(old_start, old_count,
                                    new_start, new_count,
                                    heading, line_idx, lines=[line + '\n']))
-        elif not hunks:
-            # first line of the diff is not a header line
-            errmsg = 'Malformed diff?: %s' % diff_text
-            raise AssertionError(errmsg)
-        elif line:
+        elif line and hunks:
             hunks[-1].lines.append(line + '\n')
     return hunks
 
@@ -92,7 +86,7 @@ class Counter(object):
 
     def parse(self, range_str):
         """Parse a diff range and setup internal state"""
-        start, count = _parse_range_str(range_str)
+        start, count = parse_range_str(range_str)
         self.value = start
         self.max_value = max(start + count, self.max_value)
 
@@ -126,11 +120,10 @@ class DiffLines(object):
 
     def parse(self, diff_text):
         lines = []
-        INITIAL_STATE = 0
         DIFF_STATE = 1
-        state = INITIAL_STATE
+        state = INITIAL_STATE = 0
         merge = self.merge = False
-        NO_NEWLINE = '\\ No newline at end of file'
+        NO_NEWLINE = r'\ No newline at end of file'
 
         old = self.old.reset()
         new = self.new.reset()
@@ -206,27 +199,27 @@ class FormatDigits(object):
         self.fmt = ''
         self.empty = ''
         self.dash = ''
-        self._dash = dash or compat.unichr(0xb7)
+        self._dash = dash or compat.uchr(0xb7)
         self._empty = empty or ' '
 
-    def set_digits(self, digits):
-        self.fmt = ('%%0%dd' % digits)
-        self.empty = (self._empty * digits)
-        self.dash = (self._dash * digits)
+    def set_digits(self, value):
+        self.fmt = ('%%0%dd' % value)
+        self.empty = (self._empty * value)
+        self.dash = (self._dash * value)
 
     def value(self, old, new):
         old_str = self._format(old)
         new_str = self._format(new)
-        return ('%s %s' % (old_str, new_str))
+        return '%s %s' % (old_str, new_str)
 
     def merge_value(self, old, base, new):
         old_str = self._format(old)
         base_str = self._format(base)
         new_str = self._format(new)
-        return ('%s %s %s' % (old_str, base_str, new_str))
+        return '%s %s %s' % (old_str, base_str, new_str)
 
     def number(self, value):
-        return (self.fmt % value)
+        return self.fmt % value
 
     def _format(self, value):
         if value == self.DASH:
@@ -288,7 +281,7 @@ class DiffParser(object):
                     elif line_type == DELETION:
                         line_type = ADDITION
 
-                if not (first_line_idx <= line_idx <= last_line_idx):
+                if not first_line_idx <= line_idx <= last_line_idx:
                     if line_type == ADDITION:
                         # Skip additions that are not selected.
                         prev_skipped = True
@@ -334,15 +327,15 @@ class DiffParser(object):
         # so return None.
         if len(lines) == 2:
             return None
-        else:
-            return ''.join(lines)
+        return ''.join(lines)
 
     def generate_hunk_patch(self, line_idx, reverse=False):
         """Return a patch containing the hunk for the specified line only"""
-        if not self.hunks:
-            return None
+        hunk = None
         for hunk in self.hunks:
             if line_idx <= hunk.last_line_idx:
                 break
+        if hunk is None:
+            return None
         return self.generate_patch(hunk.first_line_idx, hunk.last_line_idx,
                                    reverse=reverse)
