@@ -1,16 +1,11 @@
 """Launcher and command line interface to git-cola"""
 from __future__ import absolute_import, division, unicode_literals
 import argparse
-import os
 import sys
 
-from .app import add_common_arguments
-from .app import application_init
-from .app import application_start
+from . import app
 from . import cmds
-from . import compat
 from . import core
-from . import utils
 
 
 def main(argv=None):
@@ -21,16 +16,21 @@ def main(argv=None):
     # when none has been specified.  We fake it by injecting
     # 'cola' into the command-line so that parse_args()
     # routes them to the 'cola' parser by default.
-    if (len(argv) < 1 or
-            argv[0].startswith('-') and
-            '--help-commands' not in argv):
+    help_commands = core.encode('--help-commands')
+    args = [core.encode(arg) for arg in argv]
+    if (not argv or
+            argv[0].startswith('-') and help_commands not in args):
         argv.insert(0, 'cola')
-    elif '--help-commands' in argv:
+    elif help_commands in argv:
         argv.append('--help')
     args = parse_args(argv)
     return args.func(args)
 
 
+def winmain():
+    return app.winmain(main)
+
+
 def parse_args(argv):
     parser = argparse.ArgumentParser()
     subparser = parser.add_subparsers(title='valid commands')
@@ -41,6 +41,7 @@ def parse_args(argv):
     add_archive_command(subparser)
     add_branch_command(subparser)
     add_browse_command(subparser)
+    add_clone_command(subparser)
     add_config_command(subparser)
     add_dag_command(subparser)
     add_diff_command(subparser)
@@ -64,7 +65,7 @@ def parse_args(argv):
 def add_command(parent, name, description, func):
     parser = parent.add_parser(str(name), help=description)
     parser.set_defaults(func=func)
-    add_common_arguments(parser)
+    app.add_common_arguments(parser)
     return parser
 
 
@@ -81,6 +82,7 @@ def add_cola_command(subparser):
 def add_about_command(parent):
     add_command(parent, 'about', 'about git-cola', cmd_about)
 
+
 def add_am_command(parent):
     parser = add_command(parent, 'am', 'apply patches using "git am"', cmd_am)
     parser.add_argument('patches', metavar='<patches>', nargs='*',
@@ -99,7 +101,10 @@ def add_branch_command(subparser):
 
 def add_browse_command(subparser):
     add_command(subparser, 'browse', 'browse repository', cmd_browse)
-    add_command(subparser, 'classic', 'browse repository', cmd_browse)
+
+
+def add_clone_command(subparser):
+    add_command(subparser, 'clone', 'clone repository', cmd_clone)
 
 
 def add_config_command(subparser):
@@ -111,6 +116,8 @@ def add_dag_command(subparser):
     parser.add_argument('-c', '--count', metavar='<count>',
                         type=int, default=1000,
                         help='number of commits to display')
+    parser.add_argument('--all', action='store_true', dest='show_all',
+                        help='visualize all branches', default=False)
     parser.add_argument('args', nargs='*', metavar='<args>',
                         help='git log arguments')
 
@@ -243,6 +250,7 @@ def add_rebase_command(subparser):
                              '"git checkout <branch>" before doing anything '
                              'else when <branch> is specified')
 
+
 def add_recent_command(subparser):
     add_command(subparser, 'recent', 'edit recent files', cmd_recent)
 
@@ -270,9 +278,12 @@ def add_tag_command(subparser):
 
 
 def add_version_command(subparser):
-    parser = add_command(subparser, 'version', 'print the version', cmd_version)
+    parser = add_command(
+        subparser, 'version', 'print the version', cmd_version)
     parser.add_argument('--brief', action='store_true', default=False,
                         help='print the version number only')
+    parser.add_argument('--build', action='store_true', default=False,
+                        help='print the build version')
 
 
 # entry points
@@ -282,249 +293,243 @@ def cmd_cola(args):
     if status_filter:
         status_filter = core.abspath(status_filter)
 
-    context = application_init(args)
-    view = MainView(context.model, settings=args.settings)
+    context = app.application_init(args)
+
+    context.timer.start('view')
+    view = MainView(context, settings=args.settings)
     if args.amend:
-        cmds.do(cmds.AmendMode, True)
+        cmds.do(cmds.AmendMode, context, amend=True)
 
     if status_filter:
         view.set_filter(core.relpath(status_filter))
 
-    return application_start(context, view)
+    context.timer.stop('view')
+    if args.perf:
+        context.timer.display('view')
+
+    return app.application_run(
+        context, view, start=start_cola, stop=app.default_stop)
+
+
+def start_cola(context, view):
+    app.default_start(context, view)
+    view.start(context)
 
 
 def cmd_about(args):
     from .widgets import about
-    context = application_init(args)
-    view = about.about_dialog()
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = about.about_dialog(context)
+    return app.application_start(context, view)
 
 
 def cmd_am(args):
     from .widgets.patch import new_apply_patches
-    context = application_init(args)
-    view = new_apply_patches(patches=args.patches)
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = new_apply_patches(context, patches=args.patches)
+    return app.application_start(context, view)
 
 
 def cmd_archive(args):
-    from .widgets.archive import Archive
-    context = application_init(args, update=True)
+    from .widgets import archive
+    context = app.application_init(args, update=True)
     if args.ref is None:
         args.ref = context.model.currentbranch
-    view = Archive(args.ref)
-    return application_start(context, view)
+    view = archive.Archive(context, args.ref)
+    return app.application_start(context, view)
 
 
 def cmd_branch(args):
     from .widgets.createbranch import create_new_branch
-    context = application_init(args, update=True)
-    view = create_new_branch(settings=args.settings)
-    return application_start(context, view)
+    context = app.application_init(args, update=True)
+    view = create_new_branch(context, settings=args.settings)
+    return app.application_start(context, view)
 
 
 def cmd_browse(args):
     from .widgets.browse import worktree_browser
-    context = application_init(args)
-    view = worktree_browser(update=False, settings=args.settings)
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = worktree_browser(
+        context, show=False, update=False, settings=args.settings)
+    return app.application_start(context, view)
+
+
+def cmd_clone(args):
+    from .widgets import clone
+    context = app.application_init(args)
+    view = clone.clone(context, settings=args.settings)
+    context.set_view(view)
+    result = 0 if view.exec_() == view.Accepted else 1
+    app.default_stop(context, view)
+    return result
 
 
 def cmd_config(args):
     from .widgets.prefs import preferences
-    context = application_init(args)
-    view = preferences()
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = preferences(context)
+    return app.application_start(context, view)
 
 
 def cmd_dag(args):
-    context = application_init(args)
-    from .widgets.dag import git_dag
-    view = git_dag(context.model, args=args, settings=args.settings)
-    return application_start(context, view)
+    from .widgets import dag
+    context = app.application_init(args)
+    # cola.main() uses parse_args(), unlike dag.main() which uses
+    # parse_known_args(), thus we aren't able to automatically forward
+    # all unknown arguments.  Special-case support for "--all" since it's
+    # used by the history viewer command on Windows.
+    if args.show_all:
+        args.args.insert(0, '--all')
+    view = dag.git_dag(context, args=args, settings=args.settings, show=False)
+    return app.application_start(context, view)
 
 
 def cmd_diff(args):
-    context = application_init(args)
     from .difftool import diff_expression
+    context = app.application_init(args)
     expr = core.list2cmdline(args.args)
-    view = diff_expression(None, expr, create_widget=True)
-    return application_start(context, view)
+    view = diff_expression(context, None, expr, create_widget=True)
+    return app.application_start(context, view)
 
 
 def cmd_fetch(args):
     # TODO: the calls to update_status() can be done asynchronously
     # by hooking into the message_updated notification.
-    context = application_init(args)
     from .widgets import remote
+    context = app.application_init(args)
     context.model.update_status()
-    view = remote.fetch()
-    return application_start(context, view)
+    view = remote.fetch(context)
+    return app.application_start(context, view)
 
 
 def cmd_find(args):
-    context = application_init(args)
     from .widgets import finder
+    context = app.application_init(args)
     paths = core.list2cmdline(args.paths)
-    view = finder.finder(paths=paths)
-    return application_start(context, view)
+    view = finder.finder(context, paths=paths)
+    return app.application_start(context, view)
 
 
 def cmd_grep(args):
-    context = application_init(args)
     from .widgets import grep
+    context = app.application_init(args)
     text = core.list2cmdline(args.args)
-    view = grep.new_grep(text=text, parent=None)
-    return application_start(context, view)
+    view = grep.new_grep(context, text=text, parent=None)
+    return app.application_start(context, view)
 
 
 def cmd_merge(args):
-    context = application_init(args, update=True)
     from .widgets.merge import Merge
-    view = Merge(context.cfg, context.model, parent=None, ref=args.ref)
-    return application_start(context, view)
+    context = app.application_init(args, update=True)
+    view = Merge(context, parent=None, ref=args.ref)
+    return app.application_start(context, view)
 
 
 def cmd_version(args):
     from . import version
-    version.print_version(brief=args.brief)
+    version.print_version(brief=args.brief, build=args.build)
     return 0
 
 
 def cmd_pull(args):
     from .widgets import remote
-    context = application_init(args, update=True)
-    view = remote.pull()
+    context = app.application_init(args, update=True)
+    view = remote.pull(context)
     if args.rebase:
         view.set_rebase(True)
-    return application_start(context, view)
+    return app.application_start(context, view)
 
 
 def cmd_push(args):
     from .widgets import remote
-    context = application_init(args, update=True)
-    view = remote.push()
-    return application_start(context, view)
+    context = app.application_init(args, update=True)
+    view = remote.push(context)
+    return app.application_start(context, view)
 
 
 def cmd_rebase(args):
     kwargs = {
-            'verbose': args.verbose,
-            'quiet': args.quiet,
-            'autostash': args.autostash,
-            'fork_point': args.fork_point,
-            'onto': args.onto,
-            'preserve_merges': args.preserve_merges,
-            'strategy': args.strategy,
-            'no_ff': args.no_ff,
-            'merge': args.merge,
-            'exec': getattr(args, 'exec', None),  # python keyword
-            'keep_empty': args.keep_empty,
-            'force_rebase': args.force_rebase,
-            'strategy_option': args.strategy_option,
-            'stat': args.stat,
-            'no_stat': args.no_stat,
-            'verify': args.verify,
-            'rerere_autoupdate': args.rerere_autoupdate,
-            'root': args.root,
-            'autosquash': args.autosquash,
-            'committer_date_is_author_date': args.committer_date_is_author_date,
-            'ignore_date': args.ignore_date,
-            'whitespace': args.whitespace,
-            'ignore_whitespace': args.ignore_whitespace,
-            'C': args.context_lines,
-            'continue': getattr(args, 'continue', False),  # python keyword
-            'abort': args.abort,
-            'skip': args.skip,
-            'edit_todo': args.edit_todo,
-            'upstream': args.upstream,
-            'branch': args.branch,
+        'verbose': args.verbose,
+        'quiet': args.quiet,
+        'autostash': args.autostash,
+        'fork_point': args.fork_point,
+        'onto': args.onto,
+        'preserve_merges': args.preserve_merges,
+        'strategy': args.strategy,
+        'no_ff': args.no_ff,
+        'merge': args.merge,
+        'exec': getattr(args, 'exec', None),  # python keyword
+        'keep_empty': args.keep_empty,
+        'force_rebase': args.force_rebase,
+        'strategy_option': args.strategy_option,
+        'stat': args.stat,
+        'no_stat': args.no_stat,
+        'verify': args.verify,
+        'rerere_autoupdate': args.rerere_autoupdate,
+        'root': args.root,
+        'autosquash': args.autosquash,
+        'committer_date_is_author_date': args.committer_date_is_author_date,
+        'ignore_date': args.ignore_date,
+        'whitespace': args.whitespace,
+        'ignore_whitespace': args.ignore_whitespace,
+        'C': args.context_lines,
+        'continue': getattr(args, 'continue', False),  # python keyword
+        'abort': args.abort,
+        'skip': args.skip,
+        'edit_todo': args.edit_todo,
+        'upstream': args.upstream,
+        'branch': args.branch,
     }
-    status, out, err = cmds.do(cmds.Rebase, **kwargs)
+    context = app.application_init(args)
+    status, _, _ = cmds.do(cmds.Rebase, context, **kwargs)
     return status
 
 
 def cmd_recent(args):
     from .widgets import recent
-    context = application_init(args)
-    view = recent.browse_recent_files()
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = recent.browse_recent_files(context)
+    return app.application_start(context, view)
 
 
 def cmd_remote(args):
     from .widgets import editremotes
-    context = application_init(args)
-    view = editremotes.new_remote_editor()
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = editremotes.editor(context, run=False)
+    return app.application_start(context, view)
 
 
 def cmd_search(args):
     from .widgets.search import search
-    context = application_init(args)
-    view = search()
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = search(context)
+    return app.application_start(context, view)
 
 
 def cmd_stash(args):
-    from .widgets.stash import stash
-    context = application_init(args)
-    view = stash()
-    return application_start(context, view)
+    from .widgets import stash
+    context = app.application_init(args)
+    view = stash.view(context, show=False)
+    return app.application_start(context, view)
 
 
 def cmd_tag(args):
     from .widgets.createtag import new_create_tag
-    context = application_init(args)
-    view = new_create_tag(name=args.name, ref=args.ref, sign=args.sign,
-                          settings=args.settings)
-    return application_start(context, view)
+    context = app.application_init(args)
+    view = new_create_tag(
+        context, name=args.name, ref=args.ref, sign=args.sign,
+        settings=args.settings)
+    return app.application_start(context, view)
 
 
 # Windows shortcut launch features:
-
-def find_git():
-    """Return the path of git.exe, or None if we can't find it."""
-    if not utils.is_win32():
-        return None  # UNIX systems have git in their $PATH
-
-    # If the user wants to use a Git/bin/ directory from a non-standard
-    # directory then they can write its location into
-    # ~/.config/git-cola/git-bindir
-    git_bindir = os.path.expanduser(os.path.join('~', '.config', 'git-cola',
-                                                 'git-bindir'))
-    if core.exists(git_bindir):
-        custom_path = core.read(git_bindir).strip()
-        if custom_path and core.exists(custom_path):
-            return custom_path
-
-    # Try to find Git's bin/ directory in one of the typical locations
-    pf = os.environ.get('ProgramFiles', 'C:\\Program Files')
-    pf32 = os.environ.get('ProgramFiles(x86)', 'C:\\Program Files (x86)')
-    for p in [pf32, pf, 'C:\\']:
-        candidate = os.path.join(p, 'Git\\bin')
-        if os.path.isdir(candidate):
-            return candidate
-
-    return None
-
-
 def shortcut_launch():
     """Launch from a shortcut
 
-    Prompt for the repository by default, and try to find git.
+    Prompt for the repository by default.
+
     """
-    argv = ['cola', '--prompt']
-    git_path = find_git()
-    if git_path:
-        prepend_path(git_path)
-
-    return main(argv)
-
-
-def prepend_path(path):
-    # Adds git to the PATH.  This is needed on Windows.
-    path = core.decode(path)
-    path_entries = core.getenv('PATH', '').split(os.pathsep)
-    if path not in path_entries:
-        path_entries.insert(0, path)
-        compat.setenv('PATH', os.pathsep.join(path_entries))
+    argv = sys.argv[1:]
+    if not argv:
+        argv = ['cola', '--prompt']
+    return app.winmain(main, argv)
