@@ -1,39 +1,38 @@
-# Copyright (c) 2008 David Aguilar
+# Copyright (C) 2007-2018 David Aguilar
 """This module provides the central cola model.
 """
 from __future__ import division, absolute_import, unicode_literals
 
-import copy
 import os
 
 from .. import core
-from .. import git
 from .. import gitcmds
-from .. import gitcfg
 from ..git import STDOUT
 from ..observable import Observable
-from ..decorators import memoize
-from ..models.selection import selection_model
-from ..models import prefs
-from ..compat import ustr
+from . import prefs
 
 
-@memoize
-def model():
-    """Returns the main model singleton"""
-    return MainModel()
+def create(context):
+    """Create the repository status model"""
+    return MainModel(context)
 
 
 class MainModel(Observable):
-    """Provides a friendly wrapper for doing common git operations."""
+    """Repository status model"""
+    # TODO this class can probably be split apart into a DiffModel,
+    # CommitMessageModel, StatusModel, and an AppStatusStateMachine.
 
     # Observable messages
     message_about_to_update = 'about_to_update'
     message_commit_message_changed = 'commit_message_changed'
     message_diff_text_changed = 'diff_text_changed'
+    message_diff_text_updated = 'diff_text_updated'
+    message_diff_type_changed = 'diff_type_changed'
     message_filename_changed = 'filename_changed'
+    message_images_changed = 'images_changed'
     message_mode_about_to_change = 'mode_about_to_change'
     message_mode_changed = 'mode_changed'
+    message_submodules_changed = 'message_submodules_changed'
     message_updated = 'updated'
 
     # States
@@ -54,21 +53,24 @@ class MainModel(Observable):
     modes_unstageable = set((mode_amend, mode_index))
 
     unstaged = property(
-            lambda self: self.modified + self.unmerged + self.untracked)
+        lambda self: self.modified + self.unmerged + self.untracked)
     """An aggregate of the modified, unmerged, and untracked file lists."""
 
-    def __init__(self, cwd=None):
-        """Reads git repository settings and sets several methods
-        so that they refer to the git module.  This object
-        encapsulates cola's interaction with git."""
+    def __init__(self, context, cwd=None):
+        """Interface to the main repository status"""
         Observable.__init__(self)
 
-        # Initialize the git command object
-        self.git = git.current()
+        self.context = context
+        self.git = context.git
+        self.cfg = context.cfg
+        self.selection = context.selection
 
         self.initialized = False
+        self.annex = False
+        self.lfs = False
         self.head = 'HEAD'
         self.diff_text = ''
+        self.diff_type = 'text'  # text, image
         self.mode = self.mode_none
         self.filename = None
         self.is_merging = False
@@ -78,6 +80,7 @@ class MainModel(Observable):
         self.project = ''
         self.remotes = []
         self.filter_paths = None
+        self.images = []
 
         self.commitmsg = ''  # current commit message
         self._auto_commitmsg = ''  # e.g. .git/MERGE_MSG
@@ -91,6 +94,7 @@ class MainModel(Observable):
         self.staged_deleted = set()
         self.unstaged_deleted = set()
         self.submodules = set()
+        self.submodules_list = []
 
         self.local_branches = []
         self.remote_branches = []
@@ -113,7 +117,7 @@ class MainModel(Observable):
         return self.mode in self.modes_stageable
 
     def all_branches(self):
-        return (self.local_branches + self.remote_branches)
+        return self.local_branches + self.remote_branches
 
     def set_worktree(self, worktree):
         self.git.set_worktree(worktree)
@@ -123,9 +127,24 @@ class MainModel(Observable):
             self.project = os.path.basename(cwd)
             self.set_directory(cwd)
             core.chdir(cwd)
-            gitcfg.current().reset()
+            self.update_config(reset=True)
         return is_valid
 
+    def is_git_lfs_enabled(self):
+        """Return True if `git lfs install` has been run
+
+        We check for the existence of the "lfs" object-storea, and one of the
+        "git lfs install"-provided hooks.  This allows us to detect when
+        "git lfs uninstall" has been run.
+
+        """
+        lfs_filter = self.cfg.get('filter.lfs.clean', default=False)
+        lfs_dir = lfs_filter and self.git.git_path('lfs')
+        lfs_hook = lfs_filter and self.git.git_path('hooks', 'post-merge')
+        return (lfs_filter
+                and lfs_dir and core.exists(lfs_dir)
+                and lfs_hook and core.exists(lfs_hook))
+
     def set_commitmsg(self, msg, notify=True):
         self.commitmsg = msg
         if notify:
@@ -139,13 +158,27 @@ class MainModel(Observable):
             if not msg.endswith('\n'):
                 msg += '\n'
             core.write(path, msg)
-        except:
+        except (OSError, IOError):
             pass
         return path
 
     def set_diff_text(self, txt):
+        """Update the text displayed in the diff editor"""
+        changed = txt != self.diff_text
         self.diff_text = txt
-        self.notify_observers(self.message_diff_text_changed, txt)
+        self.notify_observers(self.message_diff_text_updated, txt)
+        if changed:
+            self.notify_observers(self.message_diff_text_changed)
+
+    def set_diff_type(self, diff_type):  # text, image
+        """Set the diff type to either text or image"""
+        self.diff_type = diff_type
+        self.notify_observers(self.message_diff_type_changed, diff_type)
+
+    def set_images(self, images):
+        """Update the images shown in the preview pane"""
+        self.images = images
+        self.notify_observers(self.message_images_changed, images)
 
     def set_directory(self, path):
         self.directory = path
@@ -169,29 +202,23 @@ class MainModel(Observable):
         self.mode = mode
         self.notify_observers(self.message_mode_changed, mode)
 
-    def apply_diff(self, filename):
-        return self.git.apply(filename, index=True, cached=True)
-
-    def apply_diff_to_worktree(self, filename):
-        return self.git.apply(filename)
-
-    def prev_commitmsg(self, *args):
-        """Queries git for the latest commit message."""
-        return self.git.log('-1', no_color=True, pretty='format:%s%n%n%b',
-                            *args)[STDOUT]
-
     def update_path_filter(self, filter_paths):
         self.filter_paths = filter_paths
         self.update_file_status()
 
-    def update_file_status(self, update_index=False):
+    def emit_about_to_update(self):
         self.notify_observers(self.message_about_to_update)
-        self._update_files(update_index=update_index)
+
+    def emit_updated(self):
         self.notify_observers(self.message_updated)
 
+    def update_file_status(self, update_index=False):
+        self.emit_about_to_update()
+        self.update_files(update_index=update_index, emit=True)
+
     def update_status(self, update_index=False):
         # Give observers a chance to respond
-        self.notify_observers(self.message_about_to_update)
+        self.emit_about_to_update()
         self.initialized = True
         self._update_merge_rebase_status()
         self._update_files(update_index=update_index)
@@ -199,14 +226,29 @@ class MainModel(Observable):
         self._update_branches_and_tags()
         self._update_branch_heads()
         self._update_commitmsg()
-        self.notify_observers(self.message_updated)
+        self.update_config()
+        self.update_submodules_list()
+        self.emit_updated()
+
+    def update_config(self, emit=False, reset=False):
+        if reset:
+            self.cfg.reset()
+        self.annex = self.cfg.is_annex()
+        self.lfs = self.is_git_lfs_enabled()
+        if emit:
+            self.emit_updated()
+
+    def update_files(self, update_index=False, emit=False):
+        self._update_files(update_index=update_index)
+        if emit:
+            self.emit_updated()
 
     def _update_files(self, update_index=False):
-        display_untracked = prefs.display_untracked()
-        state = gitcmds.worktree_state(head=self.head,
-                                       update_index=update_index,
-                                       display_untracked=display_untracked,
-                                       paths=self.filter_paths)
+        context = self.context
+        display_untracked = prefs.display_untracked(context)
+        state = gitcmds.worktree_state(
+            context, head=self.head, update_index=update_index,
+            display_untracked=display_untracked, paths=self.filter_paths)
         self.staged = state.get('staged', [])
         self.modified = state.get('modified', [])
         self.unmerged = state.get('unmerged', [])
@@ -216,12 +258,12 @@ class MainModel(Observable):
         self.unstaged_deleted = state.get('unstaged_deleted', set())
         self.submodules = state.get('submodules', set())
 
-        sel = selection_model()
+        selection = self.selection
         if self.is_empty():
-            sel.reset()
+            selection.reset()
         else:
-            sel.update(self)
-        if selection_model().is_empty():
+            selection.update(self)
+        if selection.is_empty():
             self.set_diff_text('')
 
     def is_empty(self):
@@ -236,10 +278,12 @@ class MainModel(Observable):
 
     def _update_branch_heads(self):
         # Set these early since they are used to calculate 'upstream_changed'.
-        self.currentbranch = gitcmds.current_branch()
+        self.currentbranch = gitcmds.current_branch(self.context)
 
     def _update_branches_and_tags(self):
-        local_branches, remote_branches, tags = gitcmds.all_refs(split=True)
+        context = self.context
+        local_branches, remote_branches, tags = gitcmds.all_refs(
+            context, split=True)
         self.local_branches = local_branches
         self.remote_branches = remote_branches
         self.tags = tags
@@ -261,7 +305,8 @@ class MainModel(Observable):
         if self.amending():
             return
         # Check if there's a message file in .git/
-        merge_msg_path = gitcmds.merge_message_path()
+        context = self.context
+        merge_msg_path = gitcmds.merge_message_path(context)
         if merge_msg_path:
             msg = core.read(merge_msg_path)
             if msg != self._auto_commitmsg:
@@ -273,6 +318,10 @@ class MainModel(Observable):
             self._auto_commitmsg = ''
             self.set_commitmsg(self._prev_commitmsg)
 
+    def update_submodules_list(self):
+        self.submodules_list = gitcmds.list_submodule(self.context)
+        self.notify_observers(self.message_submodules_changed)
+
     def update_remotes(self):
         self._update_remotes()
         self._update_branches_and_tags()
@@ -284,134 +333,15 @@ class MainModel(Observable):
 
     def rename_branch(self, branch, new_branch):
         status, out, err = self.git.branch(branch, new_branch, M=True)
-        self.notify_observers(self.message_about_to_update)
+        self.emit_about_to_update()
         self._update_branches_and_tags()
         self._update_branch_heads()
-        self.notify_observers(self.message_updated)
+        self.emit_updated()
         return status, out, err
 
-    def _sliced_op(self, input_items, map_fn):
-        """Slice input_items and call map_fn over every slice
-
-        This exists because of "errno: Argument list too long"
-
-        """
-        # This comment appeared near the top of include/linux/binfmts.h
-        # in the Linux source tree:
-        #
-        # /*
-        #  * MAX_ARG_PAGES defines the number of pages allocated for arguments
-        #  * and envelope for the new program. 32 should suffice, this gives
-        #  * a maximum env+arg of 128kB w/4KB pages!
-        #  */
-        # #define MAX_ARG_PAGES 32
-        #
-        # 'size' is a heuristic to keep things highly performant by minimizing
-        # the number of slices.  If we wanted it to run as few commands as
-        # possible we could call "getconf ARG_MAX" and make a better guess,
-        # but it's probably not worth the complexity (and the extra call to
-        # getconf that we can't do on Windows anyways).
-        #
-        # In my testing, getconf ARG_MAX on Mac OS X Mountain Lion reported
-        # 262144 and Debian/Linux-x86_64 reported 2097152.
-        #
-        # The hard-coded max_arg_len value is safely below both of these
-        # real-world values.
-
-        max_arg_len = 32 * 4 * 1024
-        avg_filename_len = 300
-        size = max_arg_len // avg_filename_len
-
-        status = 0
-        outs = []
-        errs = []
-
-        items = copy.copy(input_items)
-        while items:
-            stat, out, err = map_fn(items[:size])
-            status = max(stat, status)
-            outs.append(out)
-            errs.append(err)
-            items = items[size:]
-
-        return (status, '\n'.join(outs), '\n'.join(errs))
-
-    def _sliced_add(self, items):
-        add = self.git.add
-        return self._sliced_op(
-                items, lambda x: add('--', force=True, verbose=True, *x))
-
-    def stage_modified(self):
-        status, out, err = self._sliced_add(self.modified)
-        self.update_file_status()
-        return (status, out, err)
-
-    def stage_untracked(self):
-        status, out, err = self._sliced_add(self.untracked)
-        self.update_file_status()
-        return (status, out, err)
-
-    def reset(self, *items):
-        reset = self.git.reset
-        status, out, err = self._sliced_op(items, lambda x: reset('--', *x))
-        self.update_file_status()
-        return (status, out, err)
-
-    def unstage_all(self):
-        """Unstage all files, even while amending"""
-        status, out, err = self.git.reset(self.head, '--', '.')
-        self.update_file_status()
-        return (status, out, err)
-
-    def stage_all(self):
-        status, out, err = self.git.add(v=True, u=True)
-        self.update_file_status()
-        return (status, out, err)
-
-    def config_set(self, key, value, local=True):
-        # git config category.key value
-        strval = ustr(value)
-        if type(value) is bool:
-            # git uses "true" and "false"
-            strval = strval.lower()
-        if local:
-            argv = [key, strval]
-        else:
-            argv = ['--global', key, strval]
-        return self.git.config(*argv)
-
-    def config_dict(self, local=True):
-        """parses the lines from git config --list into a dictionary"""
-
-        kwargs = {
-            'list': True,
-            'global': not local,  # global is a python keyword
-        }
-        config_lines = self.git.config(**kwargs)[STDOUT].splitlines()
-        newdict = {}
-        for line in config_lines:
-            try:
-                k, v = line.split('=', 1)
-            except:
-                # value-less entry in .gitconfig
-                continue
-            k = k.replace('.', '_')  # git -> model
-            if v == 'true' or v == 'false':
-                v = bool(eval(v.title()))
-            try:
-                v = int(eval(v))
-            except:
-                pass
-            newdict[k] = v
-        return newdict
-
     def remote_url(self, name, action):
-        if action == 'push':
-            url = self.git.config('remote.%s.pushurl' % name,
-                                  get=True)[STDOUT]
-            if url:
-                return url
-        return self.git.config('remote.%s.url' % name, get=True)[STDOUT]
+        push = action == 'push'
+        return gitcmds.remote_url(self.context, name, push=push)
 
     def fetch(self, remote, **opts):
         return run_remote_action(self.git.fetch, remote, **opts)
@@ -432,7 +362,7 @@ class MainModel(Observable):
         """
         return self.git.branch(name, base, track=track, force=force)
 
-    def cherry_pick_list(self, revs, **kwargs):
+    def cherry_pick_list(self, revs):
         """Cherry-picks each revision into the current branch.
         Returns a list of command output strings (1 per cherry pick)"""
         if not revs:
@@ -447,59 +377,13 @@ class MainModel(Observable):
             errs.append(err)
         return (status, '\n'.join(outs), '\n'.join(errs))
 
-    def pad(self, pstr, num=22):
-        topad = num-len(pstr)
-        if topad > 0:
-            return pstr + ' '*topad
-        else:
-            return pstr
-
     def is_commit_published(self):
-        head = self.git.rev_parse('HEAD')[STDOUT]
-        return bool(self.git.branch(r=True, contains=head)[STDOUT])
-
-    def stage_paths(self, paths):
-        """Stages add/removals to git."""
-        if not paths:
-            self.stage_all()
-            return
-
-        add = []
-        remove = []
-
-        for path in set(paths):
-            if core.exists(path) or core.islink(path):
-                if path.endswith('/'):
-                    path = path.rstrip('/')
-                add.append(path)
-            else:
-                remove.append(path)
-
-        self.notify_observers(self.message_about_to_update)
-
-        # `git add -u` doesn't work on untracked files
-        if add:
-            self._sliced_add(add)
-
-        # If a path doesn't exist then that means it should be removed
-        # from the index.   We use `git add -u` for that.
-        if remove:
-            while remove:
-                self.git.add('--', u=True, *remove[:42])
-                remove = remove[42:]
-
-        self._update_files()
-        self.notify_observers(self.message_updated)
-
-    def unstage_paths(self, paths):
-        if not paths:
-            self.unstage_all()
-            return
-        gitcmds.unstage_paths(paths, head=self.head)
-        self.update_file_status()
+        """Return True if the latest commit exists in any remote branch"""
+        return bool(self.git.branch(r=True, contains='HEAD')[STDOUT])
 
     def untrack_paths(self, paths):
-        status, out, err = gitcmds.untrack_paths(paths, head=self.head)
+        context = self.context
+        status, out, err = gitcmds.untrack_paths(context, paths)
         self.update_file_status()
         return status, out, err
 
