@@ -1,4 +1,4 @@
-# Copyright (c) 2008 David Aguilar
+# Copyright (C) 2007-2018 David Aguilar and contributors
 """This module provides the Observable class"""
 from __future__ import division, absolute_import, unicode_literals
 
@@ -16,7 +16,7 @@ class Observable(object):
 
     def remove_observer(self, observer):
         """Remove an observer."""
-        for message, observers in self.observers.items():
+        for _, observers in self.observers.items():
             if observer in observers:
                 observers.remove(observer)
 
