@@ -1,6 +1,7 @@
-# Copyright (c) 2008-2017 David Aguilar
+# Copyright (C) 2007-2018 David Aguilar and contributors
 """Miscellaneous Qt utility functions."""
 from __future__ import division, absolute_import, unicode_literals
+import os
 
 from qtpy import compat
 from qtpy import QtGui
@@ -10,12 +11,10 @@ from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
 from . import core
-from . import gitcfg
 from . import hotkeys
 from . import icons
 from . import utils
 from .i18n import N_
-from .interaction import Interaction
 from .compat import int_types
 from .compat import ustr
 from .models import prefs
@@ -26,12 +25,9 @@ STRETCH = object()
 SKIPPED = object()
 
 
-def disconnect(signal):
-    """Disconnect signal from all slots"""
-    try:
-        signal.disconnect()
-    except TypeError:  # allow unconnected slots
-        pass
+def active_window():
+    """Return the active window for the current application"""
+    return QtWidgets.QApplication.activeWindow()
 
 
 def connect_action(action, fn):
@@ -51,6 +47,11 @@ def connect_button(button, fn):
     button.clicked.connect(lambda *args, **kwargs: fn())
 
 
+def connect_checkbox(widget, fn):
+    """Connect a checkbox to a function taking bool"""
+    widget.clicked.connect(lambda *args, **kwargs: fn(get(checkbox)))
+
+
 def connect_released(button, fn):
     """Connect a button to a function"""
     button.released.connect(fn)
@@ -66,9 +67,31 @@ def connect_toggle(toggle, fn):
     toggle.toggled.connect(fn)
 
 
-def active_window():
-    """Return the active window for the current application"""
-    return QtWidgets.QApplication.activeWindow()
+def disconnect(signal):
+    """Disconnect signal from all slots"""
+    try:
+        signal.disconnect()
+    except TypeError:  # allow unconnected slots
+        pass
+
+
+def get(widget):
+    """Query a widget for its python value"""
+    if hasattr(widget, 'isChecked'):
+        value = widget.isChecked()
+    elif hasattr(widget, 'value'):
+        value = widget.value()
+    elif hasattr(widget, 'text'):
+        value = widget.text()
+    elif hasattr(widget, 'toPlainText'):
+        value = widget.toPlainText()
+    elif hasattr(widget, 'sizes'):
+        value = widget.sizes()
+    elif hasattr(widget, 'date'):
+        value = widget.date().toString(Qt.ISODate)
+    else:
+        value = None
+    return value
 
 
 def hbox(margin, spacing, *items):
@@ -188,6 +211,25 @@ def label(text=None, align=None, fmt=None, selectable=True):
     return widget
 
 
+class ComboBox(QtWidgets.QComboBox):
+    """Custom read-only combobox with a convenient API"""
+
+    def __init__(self, items=None, editable=False, parent=None):
+        super(ComboBox, self).__init__(parent)
+        self.setEditable(editable)
+        if items:
+            self.addItems(items)
+
+    def set_index(self, idx):
+        idx = utils.clamp(idx, 0, self.count()-1)
+        self.setCurrentIndex(idx)
+
+
+def combo(items, editable=False, parent=None):
+    """Create a readonly (by default) combobox from a list of items"""
+    return ComboBox(editable=editable, items=items, parent=parent)
+
+
 def textbrowser(text=None):
     """Create a QTextBrowser for the specified text"""
     widget = QtWidgets.QTextBrowser()
@@ -205,13 +247,14 @@ def add_completer(widget, items):
     widget.setCompleter(completer)
 
 
-def prompt(msg, title=None, text=''):
+def prompt(msg, title=None, text='', parent=None):
     """Presents the user with an input widget and returns the input."""
     if title is None:
         title = msg
+    if parent is None:
+        parent = active_window()
     result = QtWidgets.QInputDialog.getText(
-            active_window(), msg, title,
-            QtWidgets.QLineEdit.Normal, text)
+        parent, title, msg, QtWidgets.QLineEdit.Normal, text)
     return (result[0], result[1])
 
 
@@ -244,7 +287,7 @@ def prompt_n(msg, inputs):
         lineedit = QtWidgets.QLineEdit()
         # Enable the OK button only when all fields have been populated
         lineedit.textChanged.connect(
-                lambda x: ok_b.setEnabled(all(get_values())))
+            lambda x: ok_b.setEnabled(all(get_values())))
         if value:
             lineedit.setText(value)
         form_widgets.append((name, lineedit))
@@ -290,7 +333,7 @@ def paths_from_indexes(model, indexes,
     return paths_from_items(items, item_type=item_type, item_filter=item_filter)
 
 
-def _true_filter(x):
+def _true_filter(_x):
     return True
 
 
@@ -304,117 +347,6 @@ def paths_from_items(items,
             if i.type() == item_type and item_filter(i)]
 
 
-def confirm(title, text, informative_text, ok_text,
-            icon=None, default=True,
-            cancel_text=None, cancel_icon=None):
-    """Confirm that an action should take place"""
-    msgbox = QtWidgets.QMessageBox(active_window())
-    msgbox.setWindowModality(Qt.WindowModal)
-    msgbox.setWindowTitle(title)
-    msgbox.setText(text)
-    msgbox.setInformativeText(informative_text)
-
-    icon = icons.mkicon(icon, icons.ok)
-    ok = msgbox.addButton(ok_text, QtWidgets.QMessageBox.ActionRole)
-    ok.setIcon(icon)
-
-    cancel = msgbox.addButton(QtWidgets.QMessageBox.Cancel)
-    cancel_icon = icons.mkicon(cancel_icon, icons.close)
-    cancel.setIcon(cancel_icon)
-    if cancel_text:
-        cancel.setText(cancel_text)
-
-    if default:
-        msgbox.setDefaultButton(ok)
-    else:
-        msgbox.setDefaultButton(cancel)
-    msgbox.exec_()
-    return msgbox.clickedButton() == ok
-
-
-class ResizeableMessageBox(QtWidgets.QMessageBox):
-
-    def __init__(self, parent):
-        QtWidgets.QMessageBox.__init__(self, parent)
-        self.setMouseTracking(True)
-        self.setSizeGripEnabled(True)
-
-    def event(self, event):
-        res = QtWidgets.QMessageBox.event(self, event)
-        event_type = event.type()
-        if (event_type == QtCore.QEvent.MouseMove or
-                event_type == QtCore.QEvent.MouseButtonPress):
-            maxi = QtCore.QSize(defs.max_size, defs.max_size)
-            self.setMaximumSize(maxi)
-            text = self.findChild(QtWidgets.QTextEdit)
-            if text is not None:
-                expand = QtWidgets.QSizePolicy.Expanding
-                text.setSizePolicy(QtWidgets.QSizePolicy(expand, expand))
-                text.setMaximumSize(maxi)
-        return res
-
-
-def critical(title, message=None, details=None):
-    """Show a warning with the provided title and message."""
-    if message is None:
-        message = title
-    mbox = ResizeableMessageBox(active_window())
-    mbox.setWindowTitle(title)
-    mbox.setTextFormat(Qt.PlainText)
-    mbox.setText(message)
-    mbox.setIcon(QtWidgets.QMessageBox.Critical)
-    mbox.setStandardButtons(QtWidgets.QMessageBox.Close)
-    mbox.setDefaultButton(QtWidgets.QMessageBox.Close)
-    if details:
-        mbox.setDetailedText(details)
-    mbox.exec_()
-
-
-def command_error(title, cmd, status, out, err):
-    """Report an error message about a failed command"""
-    details = Interaction.format_out_err(out, err)
-    message = Interaction.format_command_status(cmd, status)
-    critical(title, message=message, details=details)
-
-
-def information(title, message=None, details=None, informative_text=None):
-    """Show information with the provided title and message."""
-    if message is None:
-        message = title
-    mbox = QtWidgets.QMessageBox(active_window())
-    mbox.setStandardButtons(QtWidgets.QMessageBox.Close)
-    mbox.setDefaultButton(QtWidgets.QMessageBox.Close)
-    mbox.setWindowTitle(title)
-    mbox.setWindowModality(Qt.WindowModal)
-    mbox.setTextFormat(Qt.PlainText)
-    mbox.setText(message)
-    if informative_text:
-        mbox.setInformativeText(informative_text)
-    if details:
-        mbox.setDetailedText(details)
-    # Render into a 1-inch wide pixmap
-    pixmap = icons.cola().pixmap(defs.large_icon)
-    mbox.setIconPixmap(pixmap)
-    mbox.exec_()
-
-
-def question(title, msg, default=True):
-    """Launches a QMessageBox question with the provided title and message.
-    Passing "default=False" will make "No" the default choice."""
-    yes = QtWidgets.QMessageBox.Yes
-    no = QtWidgets.QMessageBox.No
-    buttons = yes | no
-    if default:
-        default = yes
-    else:
-        default = no
-
-    parent = active_window()
-    MessageBox = QtWidgets.QMessageBox
-    result = MessageBox.question(parent, title, msg, buttons, default)
-    return result == QtWidgets.QMessageBox.Yes
-
-
 def tree_selection(tree_item, items):
     """Returns an array of model items that correspond to the selected
     QTreeWidgetItem children"""
@@ -447,9 +379,10 @@ def selected_item(list_widget, items):
     widget_item = widget_items[0]
     row = list_widget.row(widget_item)
     if row < len(items):
-        return items[row]
+        item = items[row]
     else:
-        return None
+        item = None
+    return item
 
 
 def selected_items(list_widget, items):
@@ -519,6 +452,7 @@ def set_clipboard(text):
     persist_clipboard()
 
 
+# pylint: disable=line-too-long
 def persist_clipboard():
     """Persist the clipboard
 
@@ -529,7 +463,7 @@ def persist_clipboard():
 
     C.f. https://stackoverflow.com/questions/2007103/how-can-i-disable-clear-of-clipboard-on-exit-of-pyqt4-application
 
-    """
+    """  # noqa
     clipboard = QtWidgets.QApplication.clipboard()
     event = QtCore.QEvent(QtCore.QEvent.Clipboard)
     QtWidgets.QApplication.sendEvent(clipboard, event)
@@ -537,7 +471,8 @@ def persist_clipboard():
 
 def add_action_bool(widget, text, fn, checked, *shortcuts):
     tip = text
-    action = _add_action(widget, text, tip, fn, connect_action_bool, *shortcuts)
+    action = _add_action(widget, text, tip, fn,
+                         connect_action_bool, *shortcuts)
     action.setCheckable(True)
     action.setChecked(checked)
     return action
@@ -569,7 +504,7 @@ def _add_action(widget, text, tip, fn, connect, *shortcuts):
 
 def set_selected_item(widget, idx):
     """Sets a the currently selected item to the item at index idx."""
-    if type(widget) is QtWidgets.QTreeWidget:
+    if isinstance(widget, QtWidgets.QTreeWidget):
         item = widget.topLevelItem(idx)
         if item:
             item.setSelected(True)
@@ -617,12 +552,17 @@ def desktop():
     return app().desktop()
 
 
-def center_on_screen(widget):
-    """Move widget to the center of the default screen"""
+def desktop_size():
     desk = desktop()
     rect = desk.screenGeometry(QtGui.QCursor().pos())
-    cy = rect.height()//2
-    cx = rect.width()//2
+    return (rect.width(), rect.height())
+
+
+def center_on_screen(widget):
+    """Move widget to the center of the default screen"""
+    width, height = desktop_size()
+    cx = width // 2
+    cy = height // 2
     widget.move(cx - widget.width()//2, cy - widget.height()//2)
 
 
@@ -636,29 +576,31 @@ def default_size(parent, width, height, use_parent_height=True):
 
 
 def default_monospace_font():
-    font = QtGui.QFont()
-    family = 'Monospace'
     if utils.is_darwin():
         family = 'Monaco'
-    font.setFamily(family)
-    return font
+    else:
+        family = 'Monospace'
+    mfont = QtGui.QFont()
+    mfont.setFamily(family)
+    return mfont
 
 
-def diff_font_str():
-    font_str = gitcfg.current().get(prefs.FONTDIFF)
-    if font_str is None:
+def diff_font_str(context):
+    cfg = context.cfg
+    font_str = cfg.get(prefs.FONTDIFF)
+    if not font_str:
         font_str = default_monospace_font().toString()
     return font_str
 
 
-def diff_font():
-    return font(diff_font_str())
+def diff_font(context):
+    return font(diff_font_str(context))
 
 
 def font(string):
-    font = QtGui.QFont()
-    font.fromString(string)
-    return font
+    qfont = QtGui.QFont()
+    qfont.fromString(string)
+    return qfont
 
 
 def create_button(text='', layout=None, tooltip=None, icon=None,
@@ -666,8 +608,9 @@ def create_button(text='', layout=None, tooltip=None, icon=None,
     """Create a button, set its title, and add it to the parent."""
     button = QtWidgets.QPushButton()
     button.setCursor(Qt.PointingHandCursor)
+    button.setFocusPolicy(Qt.NoFocus)
     if text:
-        button.setText(text)
+        button.setText(' ' + text)
     if icon is not None:
         button.setIcon(icon)
         button.setIconSize(QtCore.QSize(defs.small_icon, defs.small_icon))
@@ -685,6 +628,7 @@ def create_button(text='', layout=None, tooltip=None, icon=None,
 def create_action_button(tooltip=None, icon=None):
     button = QtWidgets.QPushButton()
     button.setCursor(Qt.PointingHandCursor)
+    button.setFocusPolicy(Qt.NoFocus)
     button.setFlat(True)
     if tooltip is not None:
         button.setToolTip(tooltip)
@@ -700,8 +644,10 @@ def ok_button(text, default=True, enabled=True, icon=None):
     return create_button(text=text, icon=icon, default=default, enabled=enabled)
 
 
-def close_button():
-    return create_button(text=N_('Close'), icon=icons.close())
+def close_button(text=None, icon=None):
+    text = text or N_('Close')
+    icon = icons.mkicon(icon, icons.close)
+    return create_button(text=text, icon=icon)
 
 
 def edit_button(enabled=True, default=False):
@@ -760,9 +706,9 @@ class DockTitleBarWidget(QtWidgets.QWidget):
         QtWidgets.QWidget.__init__(self, parent)
         self.setAutoFillBackground(True)
         self.label = qlabel = QtWidgets.QLabel(title, self)
-        font = qlabel.font()
-        font.setBold(True)
-        qlabel.setFont(font)
+        qfont = qlabel.font()
+        qfont.setBold(True)
+        qlabel.setFont(qfont)
         qlabel.setCursor(Qt.OpenHandCursor)
 
         self.close_button = create_action_button(
@@ -824,12 +770,38 @@ def create_dock(title, parent, stretch=True, widget=None, fn=None):
     return dock
 
 
+def hide_dock(widget):
+    widget.toggleViewAction().setChecked(False)
+    widget.hide()
+
+
 def create_menu(title, parent):
     """Create a menu and set its title."""
-    qmenu = QtWidgets.QMenu(title, parent)
+    qmenu = DebouncingMenu(title, parent)
     return qmenu
 
 
+class DebouncingMenu(QtWidgets.QMenu):
+    """Menu that debounces mouse release action ie. stops it if occurred
+    right after menu creation.
+
+    Disables annoying behaviour when RMB is pressed to show menu, cursor is
+    moved accidentally 1px onto newly created menu and released causing to
+    execute menu action
+    """
+
+    threshold_ms = 400
+
+    def __init__(self, title, parent):
+        QtWidgets.QMenu.__init__(self, title, parent)
+        self.created_at = utils.epoch_millis()
+
+    def mouseReleaseEvent(self, event):
+        threshold = DebouncingMenu.threshold_ms
+        if (utils.epoch_millis() - self.created_at) > threshold:
+            QtWidgets.QMenu.mouseReleaseEvent(self, event)
+
+
 def add_menu(title, parent):
     """Create a menu and set its title."""
     menu = create_menu(title, parent)
@@ -842,11 +814,12 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     button.setAutoRaise(True)
     button.setAutoFillBackground(True)
     button.setCursor(Qt.PointingHandCursor)
+    button.setFocusPolicy(Qt.NoFocus)
     if icon is not None:
         button.setIcon(icon)
         button.setIconSize(QtCore.QSize(defs.small_icon, defs.small_icon))
     if text is not None:
-        button.setText(text)
+        button.setText(' ' + text)
         button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
     if tooltip is not None:
         button.setToolTip(tooltip)
@@ -855,24 +828,28 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     return button
 
 
-def mimedata_from_paths(paths):
-    """Return mimedata with a list of absolute path URLs"""
+# pylint: disable=line-too-long
+def mimedata_from_paths(context, paths):
+    """Return mimedata with a list of absolute path URLs
 
+    The text/x-moz-list format is always included by Qt, and doing
+    mimedata.removeFormat('text/x-moz-url') has no effect.
+    C.f. http://www.qtcentre.org/threads/44643-Dragging-text-uri-list-Qt-inserts-garbage
+
+    gnome-terminal expects utf-16 encoded text, but other terminals,
+    e.g. terminator, prefer utf-8, so allow cola.dragencoding
+    to override the default.
+
+    """  # noqa
+    cfg = context.cfg
     abspaths = [core.abspath(path) for path in paths]
     urls = [QtCore.QUrl.fromLocalFile(path) for path in abspaths]
 
     mimedata = QtCore.QMimeData()
     mimedata.setUrls(urls)
 
-    # The text/x-moz-list format is always included by Qt, and doing
-    # mimedata.removeFormat('text/x-moz-url') has no effect.
-    # C.f. http://www.qtcentre.org/threads/44643-Dragging-text-uri-list-Qt-inserts-garbage
-    #
-    # gnome-terminal expects utf-16 encoded text, but other terminals,
-    # e.g. terminator, prefer utf-8, so allow cola.dragencoding
-    # to override the default.
     paths_text = core.list2cmdline(abspaths)
-    encoding = gitcfg.current().get('cola.dragencoding', 'utf-16')
+    encoding = cfg.get('cola.dragencoding', 'utf-16')
     moz_text = core.encode(paths_text, encoding=encoding)
     mimedata.setData('text/x-moz-url', moz_text)
 
@@ -916,21 +893,19 @@ class Task(QtCore.QRunnable):
     def __init__(self, parent):
         QtCore.QRunnable.__init__(self)
 
-        self.channel = Channel()
+        self.channel = Channel(parent)
         self.result = None
         self.setAutoDelete(False)
 
     def run(self):
         self.result = self.task()
         self.channel.result.emit(self.result)
-        self.done()
+        self.channel.finished.emit(self)
 
+    # pylint: disable=no-self-use
     def task(self):
         return None
 
-    def done(self):
-        self.channel.finished.emit(self)
-
     def connect(self, handler):
         self.channel.result.connect(handler, type=Qt.QueuedConnection)
 
@@ -957,16 +932,17 @@ class RunTask(QtCore.QObject):
         self.tasks = []
         self.task_details = {}
         self.threadpool = QtCore.QThreadPool.globalInstance()
+        self.result_fn = None
 
-    def start(self, task, progress=None, finish=None):
+    def start(self, task, progress=None, finish=None, result=None):
         """Start the task and register a callback"""
+        self.result_fn = result
         if progress is not None:
             progress.show()
         # prevents garbage collection bugs in certain PyQt4 versions
         self.tasks.append(task)
         task_id = id(task)
-        self.task_details[task_id] = (progress, finish)
-
+        self.task_details[task_id] = (progress, finish, result)
         task.channel.finished.connect(self.finish, type=Qt.QueuedConnection)
         self.threadpool.start(task)
 
@@ -974,17 +950,20 @@ class RunTask(QtCore.QObject):
         task_id = id(task)
         try:
             self.tasks.remove(task)
-        except:
+        except ValueError:
             pass
         try:
-            progress, finish = self.task_details[task_id]
+            progress, finish, result = self.task_details[task_id]
             del self.task_details[task_id]
         except KeyError:
-            finish = progress = None
+            finish = progress = result = None
 
         if progress is not None:
             progress.hide()
 
+        if result is not None:
+            result(task.result)
+
         if finish is not None:
             finish(task)
 
@@ -1028,9 +1007,16 @@ def make_format(fg=None, bg=None, bold=False):
     return fmt
 
 
-def install():
-    Interaction.critical = staticmethod(critical)
-    Interaction.confirm = staticmethod(confirm)
-    Interaction.question = staticmethod(question)
-    Interaction.information = staticmethod(information)
-    Interaction.command_error = staticmethod(command_error)
+class ImageFormats(object):
+
+    def __init__(self):
+        # returns a list of QByteArray objects
+        formats_qba = QtGui.QImageReader.supportedImageFormats()
+        # portability: python3 data() returns bytes, python2 returns str
+        decode = core.decode
+        formats = [decode(x.data()) for x in formats_qba]
+        self.extensions = set(['.' + fmt for fmt in formats])
+
+    def ok(self, filename):
+        _, ext = os.path.splitext(filename)
+        return ext.lower() in self.extensions
