@@ -1,7 +1,6 @@
-# Copyright (c) David Aguilar
+# Copyright (C) 2007-2018 David Aguilar and contributors
 """Provide git-cola's version number"""
 from __future__ import division, absolute_import, unicode_literals
-
 import os
 import sys
 
@@ -9,13 +8,12 @@ if __name__ == '__main__':
     srcdir = os.path.dirname(os.path.dirname(__file__))
     sys.path.insert(1, srcdir)
 
-from .git import git
-from .git import STDOUT
-from .decorators import memoize
-from ._version import VERSION
+from .git import STDOUT  # noqa
+from .decorators import memoize  # noqa
+from ._version import VERSION  # noqa
 try:
     from ._build_version import BUILD_VERSION
-except:
+except ImportError:
     BUILD_VERSION = ''
 
 # minimum version requirements
@@ -32,6 +30,11 @@ _versions = {
     'check-ignore': '1.8.5',
     # git for-each-ref --sort=version:refname
     'version-sort': '2.7.0',
+    # new: git cat-file --filters --path=<path> SHA1
+    # old: git cat-file --filters blob SHA1:<path>
+    'cat-file-filters-path': '2.11.0',
+    # git rev-parse --show-superproject-working-tree was added in 2.13.0
+    'show-superproject-working-tree': '2.13.0',
 }
 
 
@@ -65,16 +68,16 @@ def check(key, ver):
     return check_version(get(key), ver)
 
 
-def check_git(key):
+def check_git(context, key):
     """Checks if Git has a specific feature"""
-    return check(key, git_version())
+    return check(key, git_version(context))
 
 
-def version_to_list(version):
+def version_to_list(value):
     """Convert a version string to a list of numbers or strings
     """
     ver_list = []
-    for p in version.split('.'):
+    for p in value.split('.'):
         try:
             n = int(p)
         except ValueError:
@@ -84,33 +87,38 @@ def version_to_list(version):
 
 
 @memoize
-def git_version_str():
+def git_version_str(context):
     """Returns the current GIT version"""
+    git = context.git
     return git.version()[STDOUT].strip()
 
 
 @memoize
-def git_version():
+def git_version(context):
     """Returns the current GIT version"""
-    parts = git_version_str().split()
+    parts = git_version_str(context).split()
     if parts and len(parts) >= 3:
-        return parts[2]
+        result = parts[2]
     else:
         # minimum supported version
-        return '1.6.3'
+        result = '1.6.3'
+    return result
 
 
-def cola_version():
-    return 'cola version %s' % version()
-
-
-def print_version(brief=False):
-    if brief:
-        msg = version()
+def cola_version(build=False):
+    if build:
+        suffix = build_version() or version()
     else:
-        msg = cola_version()
+        suffix = version()
+    return 'cola version %s' % suffix
+
+
+def print_version(brief=False, build=False):
+    if brief:
+        if build:
+            msg = build_version()
+        else:
+            msg = version()
+    else:
+        msg = cola_version(build=build)
     sys.stdout.write('%s\n' % msg)
-
-
-if __name__ == '__main__':
-    print_version(brief=True)
