@@ -1,18 +1,19 @@
 """Provides widgets related to branches"""
 from __future__ import division, absolute_import, unicode_literals
-import re
+from functools import partial
 
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
 from ..compat import odict
-from ..compat import unichr
+from ..compat import uchr
 from ..i18n import N_
 from ..interaction import Interaction
-from ..models import main
 from ..widgets import defs
 from ..widgets import standard
+from ..qtutils import get
+from .. import cmds
 from .. import gitcmds
 from .. import hotkeys
 from .. import icons
@@ -26,6 +27,22 @@ NAME_REMOTE_BRANCH = N_('Remote')
 NAME_TAGS_BRANCH = N_('Tags')
 
 
+def defer_fn(parent, title, fn, *args, **kwargs):
+    return qtutils.add_action(parent, title, partial(fn, *args, **kwargs))
+
+
+def add_branch_to_menu(menu, branch, remote_branch, remote, upstream, fn):
+    """Add a remote branch to the context menu"""
+    branch_remote, _ = gitcmds.parse_remote_branch(remote_branch)
+    if branch_remote != remote:
+        menu.addSeparator()
+    action = defer_fn(menu, remote_branch, fn, branch, remote_branch)
+    if remote_branch == upstream:
+        action.setIcon(icons.star())
+    menu.addAction(action)
+    return branch_remote
+
+
 class AsyncGitActionTask(qtutils.Task):
     """Run git action asynchronously"""
 
@@ -46,7 +63,7 @@ class AsyncGitActionTask(qtutils.Task):
 
 
 class BranchesWidget(QtWidgets.QWidget):
-    def __init__(self, titlebar, parent):
+    def __init__(self, context, parent):
         QtWidgets.QWidget.__init__(self, parent)
 
         tooltip = N_('Toggle the branches filter')
@@ -54,7 +71,7 @@ class BranchesWidget(QtWidgets.QWidget):
         self.filter_button = qtutils.create_action_button(tooltip=tooltip,
                                                           icon=icon)
 
-        self.tree = BranchesTreeWidget(parent=self)
+        self.tree = BranchesTreeWidget(context, parent=self)
         self.filter_widget = BranchesFilterWidget(self.tree)
         self.filter_widget.hide()
 
@@ -68,33 +85,32 @@ class BranchesWidget(QtWidgets.QWidget):
         self.toggle_action = qtutils.add_action(self, tooltip,
                                                 self.toggle_filter,
                                                 hotkeys.FILTER)
-
-        titlebar.add_corner_widget(self.filter_button)
         qtutils.connect_button(self.filter_button, self.toggle_filter)
 
     def toggle_filter(self):
         shown = not self.filter_widget.isVisible()
         self.filter_widget.setVisible(shown)
         if shown:
-            self.filter_widget.setFocus(True)
+            self.filter_widget.setFocus()
         else:
-            self.tree.setFocus(True)
+            self.tree.setFocus()
 
 
 class BranchesTreeWidget(standard.TreeWidget):
     updated = Signal()
 
-    def __init__(self, parent=None):
+    def __init__(self, context, parent=None):
         standard.TreeWidget.__init__(self, parent)
 
+        self.context = context
+        self.main_model = model = context.model
+
         self.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
         self.setHeaderHidden(True)
         self.setAlternatingRowColors(False)
         self.setColumnCount(1)
         self.setExpandsOnDoubleClick(False)
 
-        self.main_model = model = main.model()
-
         self.tree_helper = BranchesTreeHelper()
         self.git_helper = GitHelper(model.git)
         self.current_branch = None
@@ -108,6 +124,9 @@ class BranchesTreeWidget(standard.TreeWidget):
         # Expand items when they are clicked
         self.clicked.connect(self._toggle_expanded)
 
+        # Checkout branch when double clicked
+        self.doubleClicked.connect(self.checkout_action)
+
     def refresh(self):
         if not self._active:
             return
@@ -154,70 +173,147 @@ class BranchesTreeWidget(standard.TreeWidget):
         self.setExpanded(index, not self.isExpanded(index))
 
     def contextMenuEvent(self, event):
+        """Build and execute the context menu"""
+        context = self.context
         selected = self.selected_item()
+        if not selected:
+            return
         root = self.tree_helper.get_root(selected)
 
-        if selected.childCount() == 0 and root is not None:
-            full_name = self.tree_helper.get_full_name(selected, SLASH)
-            menu = qtutils.create_menu(N_('Actions'), self)
+        if selected.childCount() > 0 or not root:
+            return
 
-            # all branches except current item
+        full_name = self.tree_helper.get_full_name(selected, SLASH)
+        menu = qtutils.create_menu(N_('Actions'), self)
+
+        # all branches except current the current branch
+        if full_name != self.current_branch:
+            menu.addAction(qtutils.add_action(
+                menu, N_('Checkout'), self.checkout_action))
+            # remote branch
+            if NAME_REMOTE_BRANCH == root.name:
+                label = N_('Checkout as new branch')
+                action = self.checkout_new_branch_action
+                menu.addAction(qtutils.add_action(menu, label, action))
+
+            merge_menu_action = qtutils.add_action(
+                menu, N_('Merge into current branch'), self.merge_action)
+            merge_menu_action.setIcon(icons.merge())
+
+            menu.addAction(merge_menu_action)
+
+        # local and remote branch
+        if NAME_TAGS_BRANCH != root.name:
+            # local branch
+            if NAME_LOCAL_BRANCH == root.name:
+
+                remote = gitcmds.tracked_branch(context, full_name)
+                if remote is not None:
+                    menu.addSeparator()
+
+                    pull_menu_action = qtutils.add_action(
+                        menu, N_('Pull'), self.pull_action)
+                    pull_menu_action.setIcon(icons.pull())
+                    menu.addAction(pull_menu_action)
+
+                    push_menu_action = qtutils.add_action(
+                        menu, N_('Push'), self.push_action)
+                    push_menu_action.setIcon(icons.push())
+                    menu.addAction(push_menu_action)
+
+                rename_menu_action = qtutils.add_action(
+                    menu, N_('Rename Branch'), self.rename_action)
+                rename_menu_action.setIcon(icons.edit())
+
+                menu.addSeparator()
+                menu.addAction(rename_menu_action)
+
+            # not current branch
             if full_name != self.current_branch:
-                menu.addAction(qtutils.add_action(self, N_('Checkout'),
-                                                  self.checkout_action))
-                # remote branch
+                delete_label = N_('Delete Branch')
                 if NAME_REMOTE_BRANCH == root.name:
-                    label = N_('Checkout as new branch')
-                    action = self.checkout_new_branch_action
-                    menu.addAction(qtutils.add_action(self, label, action))
+                    delete_label = N_('Delete Remote Branch')
 
-                merge_menu_action = qtutils.add_action(
-                    self, N_('Merge into current branch'), self.merge_action)
-                merge_menu_action.setIcon(icons.merge())
+                delete_menu_action = qtutils.add_action(
+                    menu, delete_label, self.delete_action)
+                delete_menu_action.setIcon(icons.discard())
 
-                menu.addAction(merge_menu_action)
+                menu.addSeparator()
+                menu.addAction(delete_menu_action)
 
-            # local and remote branch
-            if NAME_TAGS_BRANCH != root.name:
-                # local branch
-                if NAME_LOCAL_BRANCH == root.name:
+        # manage upstreams for local branches
+        if root.name == NAME_LOCAL_BRANCH:
+            upstream_menu = menu.addMenu(N_('Set Upstream Branch'))
+            upstream_menu.setIcon(icons.branch())
+            self.build_upstream_menu(upstream_menu)
 
-                    remote = gitcmds.tracked_branch(full_name)
-                    if remote is not None:
-                        menu.addSeparator()
+        menu.exec_(self.mapToGlobal(event.pos()))
 
-                        pull_menu_action = qtutils.add_action(
-                            self, N_('Pull'), self.pull_action)
-                        pull_menu_action.setIcon(icons.pull())
-                        menu.addAction(pull_menu_action)
+    def build_upstream_menu(self, menu):
+        """Build the "Set Upstream Branch" sub-menu"""
+        context = self.context
+        model = self.main_model
+        selected_item = self.selected_item()
+        selected_branch = self.tree_helper.get_full_name(selected_item, SLASH)
+        remote = None
+        upstream = None
 
-                        push_menu_action = qtutils.add_action(
-                            self, N_('Push'), self.push_action)
-                        push_menu_action.setIcon(icons.push())
-                        menu.addAction(push_menu_action)
+        branches = []
+        other_branches = []
 
-                    rename_menu_action = qtutils.add_action(self,
-                                                            N_('Rename Branch'),
-                                                            self.rename_action)
-                    rename_menu_action.setIcon(icons.edit())
+        if selected_branch:
+            remote = gitcmds.upstream_remote(context, selected_branch)
+            upstream = gitcmds.tracked_branch(context, branch=selected_branch)
 
-                    menu.addSeparator()
-                    menu.addAction(rename_menu_action)
+        if not remote and 'origin' in model.remotes:
+            remote = 'origin'
 
-                # not current item
-                if full_name != self.current_branch:
-                    delete_label = N_('Delete Branch')
-                    if NAME_REMOTE_BRANCH == root.name:
-                        delete_label = N_('Delete Remote Branch')
+        if remote:
+            prefix = remote + '/'
+            for branch in model.remote_branches:
+                if branch.startswith(prefix):
+                    branches.append(branch)
+                else:
+                    other_branches.append(branch)
+        else:
+            # This can be a pretty big list, let's try to split it apart
+            branch_remote = ''
+            target = branches
+            for branch in model.remote_branches:
+                new_branch_remote, _ = gitcmds.parse_remote_branch(branch)
+                if branch_remote and branch_remote != new_branch_remote:
+                    target = other_branches
+                branch_remote = new_branch_remote
+                target.append(branch)
 
-                    delete_menu_action = qtutils.add_action(self, delete_label,
-                                                            self.delete_action)
-                    delete_menu_action.setIcon(icons.discard())
+            limit = 16
+            if not other_branches and len(branches) > limit:
+                branches, other_branches = (branches[:limit], branches[limit:])
 
-                    menu.addSeparator()
-                    menu.addAction(delete_menu_action)
+        # Add an action for each remote branch
+        current_remote = remote
 
-            menu.exec_(self.mapToGlobal(event.pos()))
+        for branch in branches:
+            current_remote = add_branch_to_menu(
+                menu, selected_branch, branch, current_remote,
+                upstream, self.set_upstream)
+
+        # This list could be longer so we tuck it away in a sub-menu.
+        # Selecting a branch from the non-default remote is less common.
+        if other_branches:
+            menu.addSeparator()
+            sub_menu = menu.addMenu(N_('Other branches'))
+            for branch in other_branches:
+                current_remote = add_branch_to_menu(
+                    sub_menu, selected_branch, branch,
+                    current_remote, upstream, self.set_upstream)
+
+    def set_upstream(self, branch, remote_branch):
+        """Configure the upstream for a branch"""
+        context = self.context
+        remote, r_branch = gitcmds.parse_remote_branch(remote_branch)
+        if remote and r_branch:
+            cmds.do(cmds.SetUpstreamBranch, context, branch, remote, r_branch)
 
     def save_tree_state(self):
         states = {}
@@ -232,15 +328,17 @@ class BranchesTreeWidget(standard.TreeWidget):
                 self.tree_helper.load_state(item, states[item.name])
 
     def update_select_branch(self):
-        item = self.tree_helper.find_child(self.topLevelItem(0),
-                                           self.current_branch)
+        context = self.context
+        current_branch = self.current_branch
+        top_item = self.topLevelItem(0)
+        item = self.tree_helper.find_child(top_item, current_branch)
+
         if item is not None:
             self.tree_helper.expand_from_item(item)
             item.setIcon(0, icons.star())
 
-            tracked_branch = gitcmds.tracked_branch(self.current_branch)
-
-            if self.current_branch is not None and tracked_branch is not None:
+            tracked_branch = gitcmds.tracked_branch(context, current_branch)
+            if current_branch and tracked_branch:
                 status = {'ahead': 0, 'behind': 0}
                 status_str = ''
 
@@ -253,10 +351,10 @@ class BranchesTreeWidget(standard.TreeWidget):
                 status['behind'] = len(log[1].splitlines())
 
                 if status['ahead'] > 0:
-                    status_str += '%s%s' % (unichr(0x2191), status['ahead'])
+                    status_str += '%s%s' % (uchr(0x2191), status['ahead'])
 
                 if status['behind'] > 0:
-                    status_str += '  %s%s' % (unichr(0x2193), status['behind'])
+                    status_str += '  %s%s' % (uchr(0x2193), status['behind'])
 
                 if status_str:
                     item.setText(0, '%s\t%s' % (item.text(0), status_str))
@@ -267,8 +365,8 @@ class BranchesTreeWidget(standard.TreeWidget):
             kwarg = {}
         task = AsyncGitActionTask(self, self.git_helper, action, args, kwarg,
                                   refresh_tree, update_remotes)
-        progress = standard.ProgressDialog(N_('Executing action %s') % action,
-                                           N_('Updating'), self)
+        progress = standard.ProgressDialog(
+            N_('Executing action %s') % action, N_('Updating'), self)
         self.runtask.start(task, progress=progress,
                            finish=self.git_action_completed)
 
@@ -278,44 +376,38 @@ class BranchesTreeWidget(standard.TreeWidget):
         if task.refresh_tree:
             self.refresh()
         if task.update_remotes:
-            model = main.model()
+            model = self.main_model
             model.update_remotes()
 
     def push_action(self):
+        context = self.context
         branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        remote_branch = gitcmds.tracked_branch(branch)
-        if remote_branch is not None:
-            rgx = re.compile(r'^(?P<remote>[^/]+)/(?P<branch>.+)$')
-            match = rgx.match(remote_branch)
-            if match:
-                remote = match.group('remote')
-                branch_name = match.group('branch')
+        remote_branch = gitcmds.tracked_branch(context, branch)
+        if remote_branch:
+            remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
+            if remote and branch_name:
                 # we assume that user wants to "Push" the selected local
                 # branch to a remote with same name
                 self.git_action_async('push', [remote, branch_name])
 
     def rename_action(self):
         branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        new_branch = qtutils.prompt(N_('Rename branch'),
-                                    N_('Enter New Branch Name'),
-                                    branch)
+        new_branch = qtutils.prompt(
+            N_('Enter New Branch Name'),
+            title=N_('Rename branch'), text=branch)
         if new_branch[1] is True and new_branch[0]:
             self.git_action_async('rename', [branch, new_branch[0]],
                                   refresh_tree=True)
 
     def pull_action(self):
+        context = self.context
         branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
-        remote_branch = gitcmds.tracked_branch(branch)
-
-        if remote_branch is not None:
-            rgx = re.compile(r'^(?P<remote>[^/]+)/(?P<branch>.+)$')
-            match = rgx.match(remote_branch)
-
-            if match:
-                remote = match.group('remote')
-                branch_name = match.group('branch')
-                self.git_action_async('pull', [remote, branch_name],
-                                      refresh_tree=True)
+        remote_branch = gitcmds.tracked_branch(context, branch)
+        if remote_branch:
+            remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
+            if remote and branch_name:
+                self.git_action_async(
+                    'pull', [remote, branch_name], refresh_tree=True)
 
     def delete_action(self):
         title = N_('Delete Branch')
@@ -325,22 +417,19 @@ class BranchesTreeWidget(standard.TreeWidget):
 
         branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
 
-        if branch != self.current_branch and qtutils.confirm(title, question,
-                                                             info, ok_btn):
+        if (branch != self.current_branch and
+                Interaction.confirm(title, question, info, ok_btn)):
             remote = False
             root = self.tree_helper.get_root(self.selected_item())
             if NAME_REMOTE_BRANCH == root.name:
                 remote = True
 
             if remote:
-                rgx = re.compile(r'^(?P<remote>[^/]+)/(?P<branch>.+)$')
-                match = rgx.match(branch)
-                if match:
-                    remote = match.group('remote')
-                    branch_name = match.group('branch')
-                    self.git_action_async('delete_remote',
-                                          [remote, branch_name],
-                                          update_remotes=True)
+                remote, branch_name = gitcmds.parse_remote_branch(branch)
+                if remote and branch_name:
+                    self.git_action_async(
+                        'delete_remote', [remote, branch_name],
+                        update_remotes=True)
             else:
                 self.git_action_async('delete_local', [branch])
 
@@ -358,8 +447,8 @@ class BranchesTreeWidget(standard.TreeWidget):
     def checkout_new_branch_action(self):
         branch = self.tree_helper.get_full_name(self.selected_item(), SLASH)
         if branch != self.current_branch:
-            new_branch = re.sub(r'^(?P<remote>[^/]+)/', '', branch)
-            self.git_action_async('checkout', [branch], {'b': new_branch})
+            _, new_branch = gitcmds.parse_remote_branch(branch)
+            self.git_action_async('checkout', ['-b', new_branch, branch])
 
 
 class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
@@ -373,7 +462,7 @@ class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
             self.setIcon(0, icon)
         self.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
 
-    # TODO: review standar.py 317.
+    # TODO: review standard.py 317.
     # original function returns 'QTreeWidgetItem' object which has no
     # attribute 'rowCount'. This workaround fix error throw when
     # navigating with keyboard and press left key
@@ -474,7 +563,7 @@ class BranchesTreeHelper(object):
 
     def load_state(self, item, state):
         """Load expanded items from a dict"""
-        if len(state.keys()) > 0:
+        if state.keys():
             item.setExpanded(True)
 
         for i in range(item.childCount()):
@@ -520,8 +609,8 @@ class GitHelper(object):
     def rename(self, branch, new_branch):
         return self.git.branch(branch, new_branch, m=True)
 
-    def checkout(self, branch, **options):
-        return self.git.checkout(branch, **options)
+    def checkout(self, *args, **options):
+        return self.git.checkout(*args, **options)
 
     @staticmethod
     def show_result(command, status, out, err):
@@ -549,7 +638,7 @@ class BranchesFilterWidget(QtWidgets.QWidget):
         self.tree.updated.connect(self.apply_filter, type=Qt.QueuedConnection)
 
     def apply_filter(self):
-        text = self.text.value()
+        text = get(self.text)
         if text == self._filter:
             return
         self._apply_bold(self._filter, False)
@@ -558,7 +647,8 @@ class BranchesFilterWidget(QtWidgets.QWidget):
             self._apply_bold(text, True)
 
     def _apply_bold(self, text, value):
-        children = self.tree.findItems(text, Qt.MatchContains | Qt.MatchRecursive)
+        match = Qt.MatchContains | Qt.MatchRecursive
+        children = self.tree.findItems(text, match)
 
         for child in children:
             if child.childCount() == 0:
