@@ -1,5 +1,5 @@
 from __future__ import division, absolute_import, unicode_literals
-import re
+from functools import partial
 
 from qtpy import QtCore
 from qtpy import QtGui
@@ -11,17 +11,16 @@ from .. import actions
 from .. import cmds
 from .. import core
 from .. import gitcmds
-from .. import gitcfg
 from .. import hotkeys
 from .. import icons
 from .. import textwrap
 from .. import qtutils
-from ..cmds import Interaction
+from ..interaction import Interaction
 from ..gitcmds import commit_message_path
 from ..i18n import N_
 from ..models import dag
 from ..models import prefs
-from ..models import selection
+from ..qtutils import get
 from ..utils import Group
 from . import defs
 from .selectcommits import select_commits
@@ -36,10 +35,10 @@ class CommitMessageEditor(QtWidgets.QWidget):
     up = Signal()
     updated = Signal()
 
-    def __init__(self, model, parent):
+    def __init__(self, context, parent):
         QtWidgets.QWidget.__init__(self, parent)
-
-        self.model = model
+        self.context = context
+        self.model = model = context.model
         self.spellcheck_initialized = False
 
         self._linebreak = None
@@ -47,21 +46,21 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self._tabwidth = None
 
         # Actions
-        self.signoff_action = qtutils.add_action(self, cmds.SignOff.name(),
-                                                 cmds.run(cmds.SignOff),
-                                                 hotkeys.SIGNOFF)
+        self.signoff_action = qtutils.add_action(
+            self, cmds.SignOff.name(), cmds.run(cmds.SignOff, context),
+            hotkeys.SIGNOFF)
         self.signoff_action.setToolTip(N_('Sign off on this commit'))
 
-        self.commit_action = qtutils.add_action(self,
-                                                N_('Commit@@verb'),
-                                                self.commit, hotkeys.COMMIT)
-        self.commit_action.setIcon(icons.download())
+        self.commit_action = qtutils.add_action(
+            self, N_('Commit@@verb'), self.commit, hotkeys.COMMIT)
+        self.commit_action.setIcon(icons.commit())
         self.commit_action.setToolTip(N_('Commit staged changes'))
-        self.clear_action = qtutils.add_action(self, N_('Clear...'), self.clear)
+        self.clear_action = qtutils.add_action(
+            self, N_('Clear...'), self.clear)
 
-        self.launch_editor = actions.launch_editor(self)
-        self.launch_difftool = actions.launch_difftool(self)
-        self.stage_or_unstage = actions.stage_or_unstage(self)
+        self.launch_editor = actions.launch_editor(context, self)
+        self.launch_difftool = actions.launch_difftool(context, self)
+        self.stage_or_unstage = actions.stage_or_unstage(context, self)
 
         self.move_up = actions.move_up(self)
         self.move_down = actions.move_down(self)
@@ -81,18 +80,23 @@ class CommitMessageEditor(QtWidgets.QWidget):
         ]
 
         # Widgets
-        self.summary = CommitSummaryLineEdit()
+        self.summary = CommitSummaryLineEdit(context)
         self.summary.setMinimumHeight(defs.tool_button_height)
         self.summary.menu_actions.extend(menu_actions)
 
-        self.description = CommitMessageTextEdit()
+        cfg = context.cfg
+        self.summary_validator = MessageValidator(context, parent=self.summary)
+        self.summary.setValidator(self.summary_validator)
+
+        self.description = CommitMessageTextEdit(context, parent=self)
+        self.description.set_dictionary(cfg.get('cola.dictionary', None))
         self.description.menu_actions.extend(menu_actions)
 
         commit_button_tooltip = N_('Commit staged changes\n'
                                    'Shortcut: Ctrl+Enter')
-        self.commit_button = qtutils.create_toolbutton(
+        self.commit_button = qtutils.create_button(
             text=N_('Commit@@verb'), tooltip=commit_button_tooltip,
-            icon=icons.download())
+            icon=icons.commit())
         self.commit_group = Group(self.commit_action, self.commit_button)
 
         self.actions_menu = qtutils.create_menu(N_('Actions'), self)
@@ -108,45 +112,46 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         # Amend checkbox
         self.amend_action = self.actions_menu.addAction(
-                N_('Amend Last Commit'))
+            N_('Amend Last Commit'))
         self.amend_action.setCheckable(True)
         self.amend_action.setShortcut(hotkeys.AMEND)
         self.amend_action.setShortcutContext(Qt.ApplicationShortcut)
 
         # Bypass hooks
         self.bypass_commit_hooks_action = self.actions_menu.addAction(
-                N_('Bypass Commit Hooks'))
+            N_('Bypass Commit Hooks'))
         self.bypass_commit_hooks_action.setCheckable(True)
         self.bypass_commit_hooks_action.setChecked(False)
 
         # Sign commits
-        cfg = gitcfg.current()
         self.sign_action = self.actions_menu.addAction(
-                N_('Create Signed Commit'))
+            N_('Create Signed Commit'))
         self.sign_action.setCheckable(True)
-        self.sign_action.setChecked(cfg.get('cola.signcommits', False))
+        signcommits = cfg.get('cola.signcommits', default=False)
+        self.sign_action.setChecked(signcommits)
 
         # Spell checker
         self.check_spelling_action = self.actions_menu.addAction(
-                N_('Check Spelling'))
+            N_('Check Spelling'))
         self.check_spelling_action.setCheckable(True)
-        self.check_spelling_action.setChecked(prefs.spellcheck())
-        self.toggle_check_spelling(prefs.spellcheck())
+        spellcheck = prefs.spellcheck(context)
+        self.check_spelling_action.setChecked(spellcheck)
+        self.toggle_check_spelling(spellcheck)
 
         # Line wrapping
         self.autowrap_action = self.actions_menu.addAction(
-                N_('Auto-Wrap Lines'))
+            N_('Auto-Wrap Lines'))
         self.autowrap_action.setCheckable(True)
-        self.autowrap_action.setChecked(prefs.linebreak())
+        self.autowrap_action.setChecked(prefs.linebreak(context))
 
         # Commit message
         self.actions_menu.addSeparator()
         self.load_commitmsg_menu = self.actions_menu.addMenu(
-                N_('Load Previous Commit Message'))
+            N_('Load Previous Commit Message'))
         self.load_commitmsg_menu.aboutToShow.connect(self.build_commitmsg_menu)
 
         self.fixup_commit_menu = self.actions_menu.addMenu(
-                N_('Fixup Previous Commit'))
+            N_('Fixup Previous Commit'))
         self.fixup_commit_menu.aboutToShow.connect(self.build_fixup_menu)
 
         self.toplayout = qtutils.hbox(defs.no_margin, defs.spacing,
@@ -162,7 +167,8 @@ class CommitMessageEditor(QtWidgets.QWidget):
         qtutils.connect_button(self.commit_button, self.commit)
 
         # Broadcast the amend mode
-        qtutils.connect_action_bool(self.amend_action, cmds.run(cmds.AmendMode))
+        qtutils.connect_action_bool(
+            self.amend_action, partial(cmds.run(cmds.AmendMode), context))
         qtutils.connect_action_bool(self.check_spelling_action,
                                     self.toggle_check_spelling)
 
@@ -175,9 +181,9 @@ class CommitMessageEditor(QtWidgets.QWidget):
         qtutils.add_action(self.summary, N_('Move Down'),
                            self.summary_cursor_down, hotkeys.DOWN)
 
-        self.selection_model = selection_model = selection.selection_model()
-        selection_model.add_observer(selection_model.message_selection_changed,
-                                     self.updated.emit)
+        self.selection = selection = context.selection
+        selection.add_observer(
+            selection.message_selection_changed, self.updated.emit)
 
         self.model.add_observer(self.model.message_commit_message_changed,
                                 self.commit_message_changed.emit)
@@ -187,8 +193,8 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         self.summary.cursor_changed.connect(self.cursor_changed.emit)
         self.description.cursor_changed.connect(
-                # description starts at line 2
-                lambda row, col: self.cursor_changed.emit(row + 2, col))
+            # description starts at line 2
+            lambda row, col: self.cursor_changed.emit(row + 2, col))
 
         self.summary.textChanged.connect(self.commit_summary_changed)
         self.description.textChanged.connect(self._commit_message_changed)
@@ -197,20 +203,21 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         self.commit_group.setEnabled(False)
 
-        self.set_tabwidth(prefs.tabwidth())
-        self.set_textwidth(prefs.textwidth())
-        self.set_linebreak(prefs.linebreak())
+        self.set_expandtab(prefs.expandtab(context))
+        self.set_tabwidth(prefs.tabwidth(context))
+        self.set_textwidth(prefs.textwidth(context))
+        self.set_linebreak(prefs.linebreak(context))
 
         # Loading message
         commit_msg = ''
-        commit_msg_path = commit_message_path()
+        commit_msg_path = commit_message_path(context)
         if commit_msg_path:
             commit_msg = core.read(commit_msg_path)
         model.set_commitmsg(commit_msg)
 
         # Allow tab to jump from the summary to the description
         self.setTabOrder(self.summary, self.description)
-        self.setFont(qtutils.diff_font())
+        self.setFont(qtutils.diff_font(context))
         self.setFocusProxy(self.summary)
 
         cfg.add_observer(cfg.message_user_config_changed, self.config_changed)
@@ -218,7 +225,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
     def config_changed(self, key, value):
         if key != prefs.SPELL_CHECK:
             return
-        if self.check_spelling_action.isChecked() == value:
+        if get(self.check_spelling_action) == value:
             return
         self.check_spelling_action.setChecked(value)
         self.toggle_check_spelling(value)
@@ -254,7 +261,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         """
         cur_position = self.summary.cursorPosition()
-        end_position = len(self.summary.value())
+        end_position = len(get(self.summary))
         if cur_position == end_position:
             self.focus_description()
         else:
@@ -262,22 +269,21 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
     def commit_message(self, raw=True):
         """Return the commit message as a unicode string"""
-        summary = self.summary.value()
+        summary = get(self.summary)
         if raw:
-            description = self.description.value()
+            description = get(self.description)
         else:
             description = self.formatted_description()
         if summary and description:
             return summary + '\n\n' + description
-        elif summary:
+        if summary:
             return summary
-        elif description:
+        if description:
             return '\n\n' + description
-        else:
-            return ''
+        return ''
 
     def formatted_description(self):
-        text = self.description.value()
+        text = get(self.description)
         if not self._linebreak:
             return text
         return textwrap.word_wrap(text, self._tabwidth, self._textwidth)
@@ -294,7 +300,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
         if '\n' in value:
             summary, description = value.split('\n', 1)
             description = description.lstrip('\n')
-            cur_description = self.description.value()
+            cur_description = get(self.description)
             if cur_description:
                 description = description + '\n' + cur_description
             # this callback is triggered by changing `summary`
@@ -303,7 +309,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
             self.description.set_value(description)
         self._commit_message_changed()
 
-    def _commit_message_changed(self, value=None):
+    def _commit_message_changed(self, _value=None):
         """Update the model when values change"""
         message = self.commit_message()
         self.model.set_commitmsg(message, notify=False)
@@ -311,7 +317,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.update_actions()
 
     def clear(self):
-        if not qtutils.confirm(
+        if not Interaction.confirm(
                 N_('Clear commit message?'),
                 N_('The commit message will be cleared.'),
                 N_('This cannot be undone.  Clear commit message?'),
@@ -320,7 +326,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.model.set_commitmsg('')
 
     def update_actions(self):
-        commit_enabled = bool(self.summary.value())
+        commit_enabled = bool(get(self.summary))
         self.commit_group.setEnabled(commit_enabled)
 
     def refresh_palettes(self):
@@ -382,6 +388,9 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         self.update_actions()
 
+    def set_expandtab(self, value):
+        self.description.set_expandtab(value)
+
     def set_tabwidth(self, width):
         self._tabwidth = width
         self.description.set_tabwidth(width)
@@ -412,7 +421,8 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
     def commit(self):
         """Attempt to create a commit from the index and commit message."""
-        if not bool(self.summary.value()):
+        context = self.context
+        if not bool(get(self.summary)):
             # Describe a good commit message
             error_msg = N_(
                 'Please supply a commit message.\n\n'
@@ -426,9 +436,9 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         msg = self.commit_message(raw=False)
 
-        # we either need to have something staged, or be merging
-        # (if there was a merge conflict resolved, there may not be anything to stage,
-        #  but we still need to commit to complete the merge)
+        # We either need to have something staged, or be merging.
+        # If there was a merge conflict resolved, there may not be anything
+        # to stage, but we still need to commit to complete the merge.
         if not (self.model.staged or self.model.is_merging):
             error_msg = N_(
                 'No changes to commit.\n\n'
@@ -436,7 +446,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
             if self.model.modified:
                 informative_text = N_('Would you like to stage and '
                                       'commit all modified files?')
-                if not qtutils.confirm(
+                if not Interaction.confirm(
                         N_('Stage and commit?'), error_msg, informative_text,
                         N_('Stage and Commit'),
                         default=True, icon=icons.save()):
@@ -444,22 +454,22 @@ class CommitMessageEditor(QtWidgets.QWidget):
             else:
                 Interaction.information(N_('Nothing to commit'), error_msg)
                 return
-            cmds.do(cmds.StageModified)
+            cmds.do(cmds.StageModified, context)
 
         # Warn that amending published commits is generally bad
-        amend = self.amend_action.isChecked()
+        amend = get(self.amend_action)
         if (amend and self.model.is_commit_published() and
-            not qtutils.confirm(
-                        N_('Rewrite Published Commit?'),
-                        N_('This commit has already been published.\n'
-                           'This operation will rewrite published history.\n'
-                           'You probably don\'t want to do this.'),
-                        N_('Amend the published commit?'),
-                        N_('Amend Commit'), default=False, icon=icons.save())):
+                not Interaction.confirm(
+                    N_('Rewrite Published Commit?'),
+                    N_('This commit has already been published.\n'
+                       'This operation will rewrite published history.\n'
+                       'You probably don\'t want to do this.'),
+                    N_('Amend the published commit?'),
+                    N_('Amend Commit'), default=False, icon=icons.save())):
             return
-        no_verify = self.bypass_commit_hooks_action.isChecked()
-        sign = self.sign_action.isChecked()
-        cmds.do(cmds.Commit, amend, msg, sign, no_verify=no_verify)
+        no_verify = get(self.bypass_commit_hooks_action)
+        sign = get(self.sign_action)
+        cmds.do(cmds.Commit, context, amend, msg, sign, no_verify=no_verify)
         self.bypass_commit_hooks_action.setChecked(False)
 
     def build_fixup_menu(self):
@@ -474,31 +484,33 @@ class CommitMessageEditor(QtWidgets.QWidget):
                                 self.choose_commit_message)
 
     def build_commits_menu(self, cmd, menu, chooser, prefix=''):
-        ctx = dag.DAG('HEAD', 6)
-        commits = dag.RepoReader(ctx)
+        context = self.context
+        params = dag.DAG('HEAD', 6)
+        commits = dag.RepoReader(context, params)
 
         menu_commits = []
-        for idx, c in enumerate(commits):
+        for idx, c in enumerate(commits.get()):
             menu_commits.insert(0, c)
             if idx > 5:
                 continue
 
         menu.clear()
         for c in menu_commits:
-            menu.addAction(prefix + c.summary, cmds.run(cmd, c.oid))
+            menu.addAction(prefix + c.summary, cmds.run(cmd, context, c.oid))
 
         if len(commits) == 6:
             menu.addSeparator()
             menu.addAction(N_('More...'), chooser)
 
     def choose_commit(self, cmd):
-        revs, summaries = gitcmds.log_helper()
-        oids = select_commits(N_('Select Commit'), revs, summaries,
-                              multiselect=False)
+        context = self.context
+        revs, summaries = gitcmds.log_helper(context)
+        oids = select_commits(
+            context, N_('Select Commit'), revs, summaries, multiselect=False)
         if not oids:
             return
         oid = oids[0]
-        cmds.do(cmd, oid)
+        cmds.do(cmd, context, oid)
 
     def choose_commit_message(self):
         self.choose_commit(cmds.LoadCommitMessageFromOID)
@@ -508,7 +520,7 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
     def toggle_check_spelling(self, enabled):
         spellcheck = self.description.spellcheck
-        cfg = gitcfg.current()
+        cfg = self.context.cfg
 
         if cfg.get_user(prefs.SPELL_CHECK) != enabled:
             cfg.set_user(prefs.SPELL_CHECK, enabled)
@@ -536,21 +548,50 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.description.highlighter.enable(enabled)
 
 
+class MessageValidator(QtGui.QValidator):
+    """Prevent invalid branch names"""
+
+    config_updated = Signal()
+
+    def __init__(self, context, parent=None):
+        super(MessageValidator, self).__init__(parent)
+        self.context = context
+        self._comment_char = None
+        self._cfg = cfg = context.cfg
+        self.refresh()
+        self.config_updated.connect(self.refresh, type=Qt.QueuedConnection)
+        cfg.add_observer(cfg.message_updated, self.emit_config_updated)
+        self.destroyed.connect(self.teardown)
+
+    def teardown(self):
+        self._cfg.remove_observer(self.emit_config_updated)
+
+    def emit_config_updated(self):
+        self.config_updated.emit()
+
+    def refresh(self):
+        """Update comment char in response to config changes"""
+        self._comment_char = prefs.comment_char(self.context)
+
+    def validate(self, string, idx):
+        """Scrub whitespace and validate the commit message"""
+        string = string.lstrip()
+        if string.startswith(self._comment_char):
+            state = self.Invalid
+        else:
+            state = self.Acceptable
+        return (state, string, idx)
+
+
 class CommitSummaryLineEdit(HintedLineEdit):
 
     cursor = Signal(int, int)
 
-    def __init__(self, parent=None):
+    def __init__(self, context, parent=None):
         hint = N_('Commit summary')
-        HintedLineEdit.__init__(self, hint, parent=parent)
+        HintedLineEdit.__init__(self, context, hint, parent=parent)
         self.menu_actions = []
 
-        comment_char = prefs.comment_char()
-        re_comment_char = re.escape(comment_char)
-        regex = QtCore.QRegExp(r'^\s*[^%s \t].*' % re_comment_char)
-        self._validator = QtGui.QRegExpValidator(regex, self)
-        self.setValidator(self._validator)
-
     def build_menu(self):
         menu = self.createStandardContextMenu()
         add_menu_actions(menu, self.menu_actions)
@@ -564,16 +605,16 @@ class CommitSummaryLineEdit(HintedLineEdit):
 class CommitMessageTextEdit(SpellCheckTextEdit):
     leave = Signal()
 
-    def __init__(self, parent=None):
+    def __init__(self, context, parent=None):
         hint = N_('Extended description...')
-        SpellCheckTextEdit.__init__(self, hint, parent)
+        SpellCheckTextEdit.__init__(self, context, hint, parent)
         self.menu_actions = []
 
         self.action_emit_leave = qtutils.add_action(
-                self, 'Shift Tab', self.leave.emit, hotkeys.LEAVE)
+            self, 'Shift Tab', self.leave.emit, hotkeys.LEAVE)
 
     def build_menu(self):
-        menu, spell_menu = self.context_menu()
+        menu, _ = self.context_menu()
         add_menu_actions(menu, self.menu_actions)
         return menu
 
