@@ -12,13 +12,65 @@ from .. import gitcmds
 from .. import icons
 from .. import qtutils
 from .. import utils
-from ..models import main
 from . import defs
-from . import text
+from .text import HintedLineEdit
 
 
-class CompletionLineEdit(text.HintedLineEdit):
-    """An lineedit with advanced completion abilities"""
+class ValidateRegex(object):
+
+    def __init__(self, regex):
+        self.regex = re.compile(regex)  # regex to scrub
+
+    def validate(self, string, idx):
+        """Scrub and validate the user-supplied input"""
+        state = QtGui.QValidator.Acceptable
+        if self.regex.search(string):
+            string = self.regex.sub('', string)  # scrub matching bits
+            idx = min(idx-1, len(string))
+        return (state, string, idx)
+
+
+class RemoteValidator(QtGui.QValidator):
+    """Prevent invalid remote names"""
+
+    def __init__(self, parent=None):
+        super(RemoteValidator, self).__init__(parent)
+        self._validate = ValidateRegex(r'[ \t\\/]')
+
+    def validate(self, string, idx):
+        return self._validate.validate(string, idx)
+
+
+class BranchValidator(QtGui.QValidator):
+    """Prevent invalid branch names"""
+
+    def __init__(self, git, parent=None):
+        super(BranchValidator, self).__init__(parent)
+        self._git = git
+        self._validate = ValidateRegex(r'[ \t\\]')  # forward-slash is okay
+
+    def validate(self, string, idx):
+        """Scrub and validate the user-supplied input"""
+        state, string, idx = self._validate.validate(string, idx)
+        if string:  # Allow empty strings
+            status, _, _ = self._git.check_ref_format(string, branch=True)
+            if status != 0:
+                # The intermediate string, when deleting characters, might
+                # end in a name that is invalid to Git, but we must allow it
+                # otherwise we won't be able to delete it using backspace.
+                if string.endswith('/') or string.endswith('.'):
+                    state = self.Intermediate
+                else:
+                    state = self.Invalid
+        return (state, string, idx)
+
+
+def _is_case_sensitive(text):
+    return bool([char for char in text if char.isupper()])
+
+
+class CompletionLineEdit(HintedLineEdit):
+    """A lineedit with advanced completion abilities"""
 
     activated = Signal()
     changed = Signal()
@@ -32,21 +84,22 @@ class CompletionLineEdit(text.HintedLineEdit):
 
     # Navigation keys trigger signals that widgets can use for customization
     NAVIGATION_KEYS = {
-            Qt.Key_Return: 'enter',
-            Qt.Key_Enter: 'enter',
-            Qt.Key_Up: 'up',
-            Qt.Key_Down: 'down',
+        Qt.Key_Return: 'enter',
+        Qt.Key_Enter: 'enter',
+        Qt.Key_Up: 'up',
+        Qt.Key_Down: 'down',
     }
 
-    def __init__(self, model_factory, hint='', parent=None):
-        text.HintedLineEdit.__init__(self, hint, parent=parent)
+    def __init__(self, context, model_factory, hint='', parent=None):
+        HintedLineEdit.__init__(self, context, hint, parent=parent)
         # Tracks when the completion popup was active during key events
-        self._was_visible = False
+
+        self.context = context
         # The most recently selected completion item
         self._selection = None
 
         # Create a completion model
-        completion_model = model_factory(self)
+        completion_model = model_factory(context, self)
         completer = Completer(completion_model, self)
         completer.setWidget(self)
         self._completer = completer
@@ -65,13 +118,10 @@ class CompletionLineEdit(text.HintedLineEdit):
     def __del__(self):
         self.dispose()
 
+    # pylint: disable=unused-argument
     def dispose(self, *args):
         self._completer.dispose()
 
-    def was_visible(self):
-        """Was the popup visible during the last keypress event?"""
-        return self._was_visible
-
     def completion_selection(self):
         """Return the last completion's selection"""
         return self._selection
@@ -88,16 +138,13 @@ class CompletionLineEdit(text.HintedLineEdit):
         """Return the completer's popup"""
         return self._completer.popup()
 
-    def _is_case_sensitive(self, text):
-        return bool([char for char in text if char.isupper()])
-
     def _text_changed(self, full_text):
         match_text = self._last_word()
         self._do_text_changed(full_text, match_text)
         self.complete_last_word()
 
     def _do_text_changed(self, full_text, match_text):
-        case_sensitive = self._is_case_sensitive(match_text)
+        case_sensitive = _is_case_sensitive(match_text)
         if case_sensitive:
             self._completer.setCaseSensitivity(Qt.CaseSensitive)
         else:
@@ -107,7 +154,7 @@ class CompletionLineEdit(text.HintedLineEdit):
 
     def update_matches(self):
         text = self._last_word()
-        case_sensitive = self._is_case_sensitive(text)
+        case_sensitive = _is_case_sensitive(text)
         self._completer.setCompletionPrefix(text)
         self._completer.model().update_matches(case_sensitive)
 
@@ -134,7 +181,8 @@ class CompletionLineEdit(text.HintedLineEdit):
         return utils.shell_split(self.value())
 
     def _ends_with_whitespace(self):
-        return self.value() != self.value().rstrip()
+        value = self.value()
+        return value != value.rstrip()
 
     def _last_word(self):
         if self._ends_with_whitespace():
@@ -146,11 +194,6 @@ class CompletionLineEdit(text.HintedLineEdit):
             return ''
         return words[-1]
 
-    def event(self, event):
-        if event.type() == QtCore.QEvent.Hide:
-            self.close_popup()
-        return text.HintedLineEdit.event(self, event)
-
     def complete_last_word(self):
         self.update_matches()
         self.complete()
@@ -159,13 +202,6 @@ class CompletionLineEdit(text.HintedLineEdit):
         if self.popup().isVisible():
             self.popup().close()
 
-    def _update_popup_items(self, prefix):
-        """
-        Filters the completer's popup items to only show items
-        with the given prefix.
-        """
-        self._completer.setCompletionPrefix(prefix)
-
     def _completions_updated(self):
         popup = self.popup()
         if not popup.isVisible():
@@ -177,6 +213,7 @@ class CompletionLineEdit(text.HintedLineEdit):
         popup.selectionModel().select(selection, mode)
 
     def selected_completion(self):
+        """Return the selected completion item"""
         popup = self.popup()
         if not popup.isVisible():
             return None
@@ -187,58 +224,61 @@ class CompletionLineEdit(text.HintedLineEdit):
         idx = indexes[0]
         item = self._completion_model.itemFromIndex(idx)
         if not item:
-            return
+            return None
         return item.text()
 
-    # Qt events
-    def keyPressEvent(self, event):
-        self._was_visible = visible = self.popup().isVisible()
-        key = event.key()
-        was_empty = not bool(self.value())
-
+    def select_completion(self):
+        """Choose the selected completion option from the completion popup"""
+        result = False
+        visible = self.popup().isVisible()
         if visible:
-            self._selection = self.selected_completion()
-        else:
-            self._selection = None
-            if event.key() in self.ACTIVATION_KEYS:
-                event.accept()
-                return
+            selection = self.selected_completion()
+            if selection:
+                self.choose_completion(selection)
+                result = True
+        return result
 
-        result = text.HintedLineEdit.keyPressEvent(self, event)
+    # Qt overrides
+    def event(self, event):
+        """Override QWidget::event() for tab completion"""
+        event_type = event.type()
 
-        # Backspace at the beginning of the line should hide the popup
-        if was_empty and visible and key == Qt.Key_Backspace:
-            self.popup().hide()
-        # Clearing a line should always emit a signal
-        is_empty = not bool(self.value())
+        if (event_type == QtCore.QEvent.KeyPress
+                and event.key() == Qt.Key_Tab
+                and self.select_completion()):
+            return True
+
+        # Make sure the popup goes away during teardown
+        if event_type == QtCore.QEvent.Hide:
+            self.close_popup()
+
+        return super(CompletionLineEdit, self).event(event)
+
+    def keyPressEvent(self, event):
+        """Process completion and navigation events"""
+        super(CompletionLineEdit, self).keyPressEvent(event)
+        visible = self.popup().isVisible()
+
+        # Hide the popup when the field is empty
+        is_empty = not self.value()
         if is_empty:
             self.cleared.emit()
-        return result
+            if visible:
+                self.popup().hide()
 
-    def keyReleaseEvent(self, event):
-        """React to release events, handle completion"""
+        # Activation keys select the completion when pressed and emit the
+        # activated signal.  Navigation keys have lower priority, and only
+        # emit when it wasn't already handled as an activation event.
         key = event.key()
-        visible = self.was_visible()
-        if not visible:
-            # If it's a navigation key then emit a signal
-            try:
-                event.accept()
-                msg = self.NAVIGATION_KEYS[key]
-                signal = getattr(self, msg)
-                signal.emit()
-                return
-            except KeyError:
-                pass
-        # Run the real release event
-        result = text.HintedLineEdit.keyReleaseEvent(self, event)
-        # If the popup was visible and we have a selected popup item
-        # then choose that completion.
-        selection = self.completion_selection()
-        if visible and selection and key in self.ACTIVATION_KEYS:
-            self.choose_completion(selection)
-            self.activated.emit()
+        if key in self.ACTIVATION_KEYS and visible:
+            if self.select_completion():
+                self.activated.emit()
             return
-        return result
+
+        navigation = self.NAVIGATION_KEYS.get(key, None)
+        if navigation:
+            signal = getattr(self, navigation)
+            signal.emit()
 
 
 class GatherCompletionsThread(QtCore.QThread):
@@ -249,17 +289,23 @@ class GatherCompletionsThread(QtCore.QThread):
         QtCore.QThread.__init__(self)
         self.model = model
         self.case_sensitive = False
+        self.running = False
+
+    def dispose(self):
+        self.running = False
+        self.wait()
 
     def run(self):
         text = None
+        self.running = True
         # Loop when the matched text changes between the start and end time.
         # This happens when gather_matches() takes too long and the
         # model's match_text changes in-between.
-        while text != self.model.match_text:
+        while self.running and text != self.model.match_text:
             text = self.model.match_text
             items = self.model.gather_matches(self.case_sensitive)
 
-        if text is not None:
+        if self.running and text is not None:
             self.items_gathered.emit(items)
 
 
@@ -273,10 +319,9 @@ class HighlightDelegate(QtWidgets.QStyledItemDelegate):
         self.case_sensitive = False
 
         self.doc = QtGui.QTextDocument()
-        try:
+        # older PyQt4 does not have setDocumentMargin
+        if hasattr(self.doc, 'setDocumentMargin'):
             self.doc.setDocumentMargin(0)
-        except:  # older PyQt4
-            pass
 
     def set_highlight_text(self, text, case_sensitive):
         """Sets the text that will be made bold when displayed"""
@@ -286,8 +331,8 @@ class HighlightDelegate(QtWidgets.QStyledItemDelegate):
     def paint(self, painter, option, index):
         """Overloaded Qt method for custom painting of a model index"""
         if not self.highlight_text:
-            return QtWidgets.QStyledItemDelegate.paint(self, painter,
-                                                       option, index)
+            QtWidgets.QStyledItemDelegate.paint(self, painter, option, index)
+            return
         text = index.data()
         if self.case_sensitive:
             html = text.replace(self.highlight_text,
@@ -314,7 +359,7 @@ class HighlightDelegate(QtWidgets.QStyledItemDelegate):
         ctx = QtGui.QAbstractTextDocumentLayout.PaintContext()
 
         # Highlighting text if item is selected
-        if (params.state & QtWidgets.QStyle.State_Selected):
+        if params.state & QtWidgets.QStyle.State_Selected:
             color = params.palette.color(QtGui.QPalette.Active,
                                          QtGui.QPalette.HighlightedText)
             ctx.palette.setColor(QtGui.QPalette.Text, color)
@@ -345,8 +390,9 @@ class CompletionModel(QtGui.QStandardItemModel):
     items_gathered = Signal(object)
     model_updated = Signal()
 
-    def __init__(self, parent):
+    def __init__(self, context, parent):
         QtGui.QStandardItemModel.__init__(self, parent)
+        self.context = context
         self.match_text = ''
         self.full_text = ''
         self.case_sensitive = False
@@ -370,6 +416,7 @@ class CompletionModel(QtGui.QStandardItemModel):
         if not self.update_thread.isRunning():
             self.update_thread.start()
 
+    # pylint: disable=unused-argument,no-self-use
     def gather_matches(self, case_sensitive):
         return ((), (), set())
 
@@ -397,9 +444,15 @@ class CompletionModel(QtGui.QStandardItemModel):
                 item.setIcon(from_filename(match))
             items.append(item)
 
-        self.clear()
-        self.invisibleRootItem().appendRows(items)
-        self.updated.emit()
+        try:
+            self.clear()
+            self.invisibleRootItem().appendRows(items)
+            self.updated.emit()
+        except RuntimeError:  # C++ object has been deleted
+            pass
+
+    def dispose(self):
+        self.update_thread.dispose()
 
 
 def _identity(x):
@@ -463,9 +516,9 @@ class Completer(QtWidgets.QCompleter):
 
 class GitCompletionModel(CompletionModel):
 
-    def __init__(self, parent):
-        CompletionModel.__init__(self, parent)
-        self.main_model = model = main.model()
+    def __init__(self, context, parent):
+        CompletionModel.__init__(self, context, parent)
+        self.main_model = model = context.model
         msg = model.message_updated
         model.add_observer(msg, self.emit_model_updated)
 
@@ -480,49 +533,63 @@ class GitCompletionModel(CompletionModel):
         except RuntimeError:  # C++ object has been deleted
             self.dispose()
 
+    # pylint: disable=no-self-use
     def matches(self):
         return []
 
     def dispose(self):
+        super(GitCompletionModel, self).dispose()
         self.main_model.remove_observer(self.emit_model_updated)
 
 
 class GitRefCompletionModel(GitCompletionModel):
     """Completer for branches and tags"""
 
-    def __init__(self, parent):
-        GitCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitCompletionModel.__init__(self, context, parent)
 
     def matches(self):
         model = self.main_model
         return model.local_branches + model.remote_branches + model.tags
 
 
-class GitPotentialBranchCompletionModel(GitCompletionModel):
-    """Completer for branches, tags, and potential branches"""
+def find_potential_branches(model):
+    remotes = model.remotes
+    remote_branches = model.remote_branches
 
-    def __init__(self, parent):
-        GitCompletionModel.__init__(self, parent)
+    ambiguous = set()
+    allnames = set(model.local_branches)
+    potential = []
+
+    for remote_branch in remote_branches:
+        branch = gitcmds.strip_remote(remotes, remote_branch)
+        if branch in allnames or branch == remote_branch:
+            ambiguous.add(branch)
+            continue
+        potential.append(branch)
+        allnames.add(branch)
+
+    potential_branches = [p for p in potential if p not in ambiguous]
+    return potential_branches
+
+
+class GitCreateBranchCompletionModel(GitCompletionModel):
+    """Completer for naming new branches"""
 
     def matches(self):
         model = self.main_model
-        remotes = model.remotes
-        remote_branches = model.remote_branches
+        potential_branches = find_potential_branches(model)
+        return (model.local_branches +
+                potential_branches +
+                model.tags)
 
-        ambiguous = set()
-        allnames = set(model.local_branches)
-        potential = []
 
-        for remote_branch in remote_branches:
-            branch = gitcmds.strip_remote(remotes, remote_branch)
-            if branch in allnames or branch == remote_branch:
-                ambiguous.add(branch)
-                continue
-            potential.append(branch)
-            allnames.add(branch)
-
-        potential_branches = [p for p in potential if p not in ambiguous]
+class GitCheckoutBranchCompletionModel(GitCompletionModel):
+    """Completer for git checkout <branch>"""
 
+    def matches(self):
+        model = self.main_model
+        potential_branches = find_potential_branches(model)
         return (model.local_branches +
                 potential_branches +
                 model.remote_branches +
@@ -530,10 +597,10 @@ class GitPotentialBranchCompletionModel(GitCompletionModel):
 
 
 class GitBranchCompletionModel(GitCompletionModel):
-    """Completer for remote branches"""
+    """Completer for local branches"""
 
-    def __init__(self, parent):
-        GitCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitCompletionModel.__init__(self, context, parent)
 
     def matches(self):
         model = self.main_model
@@ -543,8 +610,8 @@ class GitBranchCompletionModel(GitCompletionModel):
 class GitRemoteBranchCompletionModel(GitCompletionModel):
     """Completer for remote branches"""
 
-    def __init__(self, parent):
-        GitCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitCompletionModel.__init__(self, context, parent)
 
     def matches(self):
         model = self.main_model
@@ -554,9 +621,10 @@ class GitRemoteBranchCompletionModel(GitCompletionModel):
 class GitPathCompletionModel(GitCompletionModel):
     """Base class for path completion"""
 
-    def __init__(self, parent):
-        GitCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitCompletionModel.__init__(self, context, parent)
 
+    # pylint: disable=no-self-use
     def candidate_paths(self):
         return []
 
@@ -570,8 +638,8 @@ class GitPathCompletionModel(GitCompletionModel):
 class GitStatusFilterCompletionModel(GitPathCompletionModel):
     """Completer for modified files and folders for status filtering"""
 
-    def __init__(self, parent):
-        GitPathCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitPathCompletionModel.__init__(self, context, parent)
 
     def candidate_paths(self):
         model = self.main_model
@@ -582,13 +650,14 @@ class GitStatusFilterCompletionModel(GitPathCompletionModel):
 class GitTrackedCompletionModel(GitPathCompletionModel):
     """Completer for tracked files and folders"""
 
-    def __init__(self, parent):
-        GitPathCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitPathCompletionModel.__init__(self, context, parent)
         self.model_updated.connect(self.gather_paths, type=Qt.QueuedConnection)
         self._paths = []
 
     def gather_paths(self):
-        self._paths = gitcmds.tracked_files()
+        context = self.context
+        self._paths = gitcmds.tracked_files(context)
 
     def gather_matches(self, case_sensitive):
         if not self._paths:
@@ -603,13 +672,14 @@ class GitTrackedCompletionModel(GitPathCompletionModel):
 class GitLogCompletionModel(GitRefCompletionModel):
     """Completer for arguments suitable for git-log like commands"""
 
-    def __init__(self, parent):
-        GitRefCompletionModel.__init__(self, parent)
+    def __init__(self, context, parent):
+        GitRefCompletionModel.__init__(self, context, parent)
         self.model_updated.connect(self.gather_paths, type=Qt.QueuedConnection)
         self._paths = []
 
     def gather_paths(self):
-        self._paths = gitcmds.tracked_files()
+        context = self.context
+        self._paths = gitcmds.tracked_files(context)
 
     def gather_matches(self, case_sensitive):
         if not self._paths:
@@ -634,9 +704,10 @@ def bind_lineedit(model, hint=''):
 
     class BoundLineEdit(CompletionLineEdit):
 
-        def __init__(self, hint=hint, parent=None):
-            CompletionLineEdit.__init__(self, model,
+        def __init__(self, context, hint=hint, parent=None):
+            CompletionLineEdit.__init__(self, context, model,
                                         hint=hint, parent=parent)
+            self.context = context
 
     return BoundLineEdit
 
@@ -644,8 +715,10 @@ def bind_lineedit(model, hint=''):
 # Concrete classes
 GitLogLineEdit = bind_lineedit(GitLogCompletionModel, hint='<ref>')
 GitRefLineEdit = bind_lineedit(GitRefCompletionModel, hint='<ref>')
-GitPotentialBranchLineEdit = bind_lineedit(GitPotentialBranchCompletionModel,
-                                           hint='<branch>')
+GitCheckoutBranchLineEdit = bind_lineedit(GitCheckoutBranchCompletionModel,
+                                          hint='<branch>')
+GitCreateBranchLineEdit = bind_lineedit(GitCreateBranchCompletionModel,
+                                        hint='<branch>')
 GitBranchLineEdit = bind_lineedit(GitBranchCompletionModel, hint='<branch>')
 GitRemoteBranchLineEdit = bind_lineedit(GitRemoteBranchCompletionModel,
                                         hint='<remote-branch>')
@@ -656,15 +729,16 @@ GitTrackedLineEdit = bind_lineedit(GitTrackedCompletionModel, hint='<path>')
 
 class GitDialog(QtWidgets.QDialog):
 
-    def __init__(self, lineedit, title, text, parent, icon=None):
+    def __init__(self, lineedit, context, title, text, parent, icon=None):
         QtWidgets.QDialog.__init__(self, parent)
+        self.context = context
         self.setWindowTitle(title)
         self.setWindowModality(Qt.WindowModal)
         self.setMinimumWidth(333)
 
         self.label = QtWidgets.QLabel()
         self.label.setText(title)
-        self.lineedit = lineedit()
+        self.lineedit = lineedit(context)
         self.ok_button = qtutils.ok_button(text, icon=icon, enabled=False)
         self.close_button = qtutils.close_button()
 
@@ -683,25 +757,24 @@ class GitDialog(QtWidgets.QDialog):
         qtutils.connect_button(self.close_button, self.reject)
 
         self.setFocusProxy(self.lineedit)
-        self.lineedit.setFocus(True)
+        self.lineedit.setFocus()
 
     def text(self):
         return self.lineedit.text()
 
-    def text_changed(self, txt):
+    def text_changed(self, _txt):
         self.ok_button.setEnabled(bool(self.text()))
 
     def set_text(self, ref):
         self.lineedit.setText(ref)
 
     @classmethod
-    def get(cls, title, text, parent, default=None, icon=None):
-        dlg = cls(title, text, parent, icon=icon)
+    def get(cls, context, title, text, parent, default=None, icon=None):
+        dlg = cls(context, title, text, parent, icon=icon)
         if default:
             dlg.set_text(default)
 
         dlg.show()
-        dlg.raise_()
 
         def show_popup():
             x = dlg.lineedit.x()
@@ -711,38 +784,38 @@ class GitDialog(QtWidgets.QDialog):
             dlg.lineedit.popup().move(mapped.x(), mapped.y())
             dlg.lineedit.popup().show()
             dlg.lineedit.refresh()
+            dlg.lineedit.setFocus()
 
-        QtCore.QTimer().singleShot(0, show_popup)
+        QtCore.QTimer().singleShot(100, show_popup)
 
         if dlg.exec_() == cls.Accepted:
             return dlg.text()
-        else:
-            return None
+        return None
 
 
 class GitRefDialog(GitDialog):
 
-    def __init__(self, title, text, parent, icon=None):
-        GitDialog.__init__(self, GitRefLineEdit,
-                           title, text, parent, icon=icon)
+    def __init__(self, context, title, text, parent, icon=None):
+        GitDialog.__init__(
+            self, GitRefLineEdit, context, title, text, parent, icon=icon)
 
 
-class GitPotentialBranchDialog(GitDialog):
+class GitCheckoutBranchDialog(GitDialog):
 
-    def __init__(self, title, text, parent, icon=None):
-        GitDialog.__init__(self, GitPotentialBranchLineEdit,
-                           title, text, parent, icon=icon)
+    def __init__(self, context, title, text, parent, icon=None):
+        GitDialog.__init__(self, GitCheckoutBranchLineEdit,
+                           context, title, text, parent, icon=icon)
 
 
 class GitBranchDialog(GitDialog):
 
-    def __init__(self, title, text, parent, icon=None):
-        GitDialog.__init__(self, GitBranchLineEdit,
-                           title, text, parent, icon=icon)
+    def __init__(self, context, title, text, parent, icon=None):
+        GitDialog.__init__(
+            self, GitBranchLineEdit, context, title, text, parent, icon=icon)
 
 
 class GitRemoteBranchDialog(GitDialog):
 
-    def __init__(self, title, text, parent, icon=None):
+    def __init__(self, context, title, text, parent, icon=None):
         GitDialog.__init__(self, GitRemoteBranchLineEdit,
-                           title, text, parent, icon=icon)
+                           context, title, text, parent, icon=icon)
