@@ -2,12 +2,12 @@ from __future__ import division, absolute_import, unicode_literals
 
 from qtpy import QtWidgets
 from qtpy.QtCore import Signal
+from qtpy.QtCore import QSize
 
 from .. import cmds
 from .. import hotkeys
 from .. import qtutils
 from ..i18n import N_
-from ..git import git
 from .standard import TreeWidget
 from .diff import COMMITS_SELECTED
 from .diff import FILES_SELECTED
@@ -20,38 +20,43 @@ class FileWidget(TreeWidget):
 
     grab_file = Signal(object)
 
-    def __init__(self, notifier, parent):
+    def __init__(self, context, notifier, parent):
         TreeWidget.__init__(self, parent)
+        self.context = context
         self.notifier = notifier
-        self.setHeaderLabels([N_('Filename'), N_('Additions'), N_('Deletions')])
+
+        labels = [N_('Filename'), N_('Additions'), N_('Deletions')]
+        self.setHeaderLabels(labels)
+
         notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
 
         self.show_history_action = qtutils.add_action(
-                self, N_('Show History'), self.show_history, hotkeys.HISTORY)
+            self, N_('Show History'), self.show_history, hotkeys.HISTORY)
 
         self.launch_difftool_action = qtutils.add_action(
-                self, N_('Launch Diff Tool'), self.show_diff)
+            self, N_('Launch Diff Tool'), self.show_diff)
 
         self.launch_editor_action = qtutils.add_action(
-                self, N_('Launch Editor'), self.edit_paths, hotkeys.EDIT)
+            self, N_('Launch Editor'), self.edit_paths, hotkeys.EDIT)
 
         self.grab_file_action = qtutils.add_action(
-                self, N_('Grab File...'), self._grab_file)
+            self, N_('Grab File...'), self._grab_file)
 
         self.itemSelectionChanged.connect(self.selection_changed)
 
     def selection_changed(self):
         items = self.selected_items()
-        self.notifier.notify_observers(FILES_SELECTED,
-                                       [i.path for i in items])
+        self.notifier.notify_observers(
+            FILES_SELECTED, [i.path for i in items])
 
     def commits_selected(self, commits):
         if not commits:
             return
+        git = self.context.git
         commit = commits[0]
         oid = commit.oid
-        status, out, err = git.show(oid, z=True, numstat=True,
-                                    oneline=True, no_renames=True)
+        status, out, _ = git.show(oid, z=True, numstat=True,
+                                  oneline=True, no_renames=True)
         if status == 0:
             paths = [f for f in out.rstrip('\0').split('\0') if f]
             if paths:
@@ -70,19 +75,23 @@ class FileWidget(TreeWidget):
             files.append(item)
         self.insertTopLevelItems(0, files)
 
-    def adjust_columns(self):
-        width = self.width() - 20
-        zero = width*2 // 3
-        onetwo = width // 6
-        self.setColumnWidth(0, zero)
-        self.setColumnWidth(1, onetwo)
-        self.setColumnWidth(2, onetwo)
+    def adjust_columns(self, size, old_size):
+        if size.isValid() and old_size.isValid():
+            width = (self.columnWidth(0) + size.width() - old_size.width())
+            self.setColumnWidth(0, width)
+        else:
+            width = self.width()
+            two_thirds = (width * 2) // 3
+            one_sixth = width // 6
+            self.setColumnWidth(0, two_thirds)
+            self.setColumnWidth(1, one_sixth)
+            self.setColumnWidth(2, one_sixth)
 
     def show(self):
-        self.adjust_columns()
+        self.adjust_columns(QSize(), QSize())
 
     def resizeEvent(self, e):
-        self.adjust_columns()
+        self.adjust_columns(e.size(), e.oldSize())
 
     def contextMenuEvent(self, event):
         menu = qtutils.create_menu(N_('Actions'), self)
@@ -103,12 +112,12 @@ class FileWidget(TreeWidget):
         return [i.path for i in self.selected_items()]
 
     def edit_paths(self):
-        cmds.do(cmds.Edit, self.selected_paths())
+        cmds.do(cmds.Edit, self.context, self.selected_paths())
 
     def show_history(self):
         items = self.selected_items()
-        self.notifier.notify_observers(HISTORIES_SELECTED,
-                                       [i.path for i in items])
+        paths = [i.path for i in items]
+        self.notifier.notify_observers(HISTORIES_SELECTED, paths)
 
 
 class FileTreeWidgetItem(QtWidgets.QTreeWidgetItem):
