@@ -1,30 +1,34 @@
 from __future__ import division, absolute_import, unicode_literals
 import os
+import re
 
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 
 from ..i18n import N_
+from ..qtutils import get
 from .. import core
 from .. import cmds
 from .. import hotkeys
+from .. import observable
 from .. import icons
 from .. import qtutils
 from .standard import Dialog
 from .standard import DraggableTreeWidget
 from . import defs
+from . import diff
 
 
-def apply_patches():
+def apply_patches(context):
     parent = qtutils.active_window()
-    dlg = new_apply_patches(parent=parent)
+    dlg = new_apply_patches(context, parent=parent)
     dlg.show()
     dlg.raise_()
     return dlg
 
 
-def new_apply_patches(patches=None, parent=None):
-    dlg = ApplyPatches(parent=parent)
+def new_apply_patches(context, patches=None, parent=None):
+    dlg = ApplyPatches(context, parent=parent)
     if patches:
         dlg.add_paths(patches)
     return dlg
@@ -53,7 +57,7 @@ def get_patches_from_mimedata(mimedata):
 def get_patches_from_dir(path):
     """Find patches in a subdirectory"""
     patches = []
-    for root, subdirs, files in core.walk(path):
+    for root, _, files in core.walk(path):
         for name in [f for f in files if f.endswith('.patch')]:
             patches.append(core.decode(os.path.join(root, name)))
     return patches
@@ -61,8 +65,9 @@ def get_patches_from_dir(path):
 
 class ApplyPatches(Dialog):
 
-    def __init__(self, parent=None):
+    def __init__(self, context, parent=None):
         super(ApplyPatches, self).__init__(parent=parent)
+        self.context = context
         self.setWindowTitle(N_('Apply Patches'))
         self.setAcceptDrops(True)
         if parent is not None:
@@ -81,26 +86,31 @@ class ApplyPatches(Dialog):
 
         self.tree = PatchTreeWidget(parent=self)
         self.tree.setHeaderHidden(True)
+        self.tree.itemSelectionChanged.connect(self._tree_selection_changed)
+
+        self.notifier = notifier = observable.Observable()
+        self.diffwidget = diff.DiffWidget(context, notifier, self,
+                                          is_commit=True)
 
         self.add_button = qtutils.create_toolbutton(
-                text=N_('Add'), icon=icons.add(),
-                tooltip=N_('Add patches (+)'))
+            text=N_('Add'), icon=icons.add(),
+            tooltip=N_('Add patches (+)'))
 
         self.remove_button = qtutils.create_toolbutton(
-                text=N_('Remove'), icon=icons.remove(),
-                tooltip=N_('Remove selected (Delete)'))
+            text=N_('Remove'), icon=icons.remove(),
+            tooltip=N_('Remove selected (Delete)'))
 
         self.apply_button = qtutils.create_button(
-                text=N_('Apply'), icon=icons.ok())
+            text=N_('Apply'), icon=icons.ok())
 
         self.close_button = qtutils.close_button()
 
         self.add_action = qtutils.add_action(
-                self, N_('Add'), self.add_files, hotkeys.ADD_ITEM)
+            self, N_('Add'), self.add_files, hotkeys.ADD_ITEM)
 
         self.remove_action = qtutils.add_action(
-                self, N_('Remove'), self.tree.remove_selected,
-                hotkeys.DELETE, hotkeys.BACKSPACE, hotkeys.REMOVE_ITEM)
+            self, N_('Remove'), self.tree.remove_selected,
+            hotkeys.DELETE, hotkeys.BACKSPACE, hotkeys.REMOVE_ITEM)
 
         self.top_layout = qtutils.hbox(defs.no_margin, defs.button_spacing,
                                        self.add_button, self.remove_button,
@@ -110,8 +120,11 @@ class ApplyPatches(Dialog):
                                           self.close_button, qtutils.STRETCH,
                                           self.apply_button)
 
+        self.splitter = qtutils.splitter(Qt.Vertical,
+                                         self.tree, self.diffwidget)
+
         self.main_layout = qtutils.vbox(defs.margin, defs.spacing,
-                                        self.top_layout, self.tree,
+                                        self.top_layout, self.splitter,
                                         self.bottom_layout)
         self.setLayout(self.main_layout)
 
@@ -126,8 +139,9 @@ class ApplyPatches(Dialog):
         items = self.tree.items()
         if not items:
             return
+        context = self.context
         patches = [i.data(0, Qt.UserRole) for i in items]
-        cmds.do(cmds.ApplyPatches, patches)
+        cmds.do(cmds.ApplyPatches, context, patches)
         self.accept()
 
     def add_files(self):
@@ -137,7 +151,7 @@ class ApplyPatches(Dialog):
         if not files:
             return
         self.curdir = os.path.dirname(files[0])
-        self.add_paths([os.path.relpath(f) for f in files])
+        self.add_paths([core.relpath(f) for f in files])
 
     def dragEnterEvent(self, event):
         """Accepts drops if the mimedata contains patches"""
@@ -157,6 +171,34 @@ class ApplyPatches(Dialog):
     def add_paths(self, paths):
         self.tree.add_paths(paths)
 
+    def _tree_selection_changed(self):
+        items = self.tree.selected_items()
+        if not items:
+            return
+        item = items[-1]  # take the last item
+        path = item.data(0, Qt.UserRole)
+        if not core.exists(path):
+            return
+        commit = parse_patch(path)
+        self.diffwidget.set_details(commit.oid, commit.author, commit.email,
+                                    commit.date, commit.summary)
+        self.diffwidget.set_diff(commit.diff)
+
+    def export_state(self):
+        """Export persistent settings"""
+        state = super(ApplyPatches, self).export_state()
+        state['sizes'] = get(self.splitter)
+        return state
+
+    def apply_state(self, state):
+        """Apply persistent settings"""
+        result = super(ApplyPatches, self).apply_state(state)
+        try:
+            self.splitter.setSizes(state['sizes'])
+        except (AttributeError, KeyError, ValueError, TypeError):
+            pass
+        return result
+
 
 class PatchTreeWidget(DraggableTreeWidget):
 
@@ -182,3 +224,57 @@ class PatchTreeWidget(DraggableTreeWidget):
         rows = [idx.row() for idx in idxs]
         for row in reversed(sorted(rows)):
             self.invisibleRootItem().takeChild(row)
+
+
+class Commit(object):
+    """Container for commit details"""
+
+    def __init__(self):
+        self.content = ''
+        self.author = ''
+        self.email = ''
+        self.oid = ''
+        self.summary = ''
+        self.diff = ''
+        self.date = ''
+
+
+def parse_patch(path):
+    content = core.read(path)
+    commit = Commit()
+    parse(content, commit)
+    return commit
+
+
+def parse(content, commit):
+    """Parse commit details from a patch"""
+    from_rgx = re.compile(r'^From (?P<oid>[a-f0-9]{40}) .*$')
+    author_rgx = re.compile(r'^From: (?P<author>[^<]+) <(?P<email>[^>]+)>$')
+    date_rgx = re.compile(r'^Date: (?P<date>.*)$')
+    subject_rgx = re.compile(r'^Subject: (?P<summary>.*)$')
+
+    commit.content = content
+
+    lines = content.splitlines()
+    for idx, line in enumerate(lines):
+        match = from_rgx.match(line)
+        if match:
+            commit.oid = match.group('oid')
+            continue
+
+        match = author_rgx.match(line)
+        if match:
+            commit.author = match.group('author')
+            commit.email = match.group('email')
+            continue
+
+        match = date_rgx.match(line)
+        if match:
+            commit.date = match.group('date')
+            continue
+
+        match = subject_rgx.match(line)
+        if match:
+            commit.summary = match.group('summary')
+            commit.diff = '\n'.join(lines[idx + 1:])
+            break
