@@ -1,6 +1,6 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import time
+from functools import partial
 
 from qtpy import QtCore
 from qtpy import QtGui
@@ -9,9 +9,13 @@ from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 from qtpy.QtWidgets import QDockWidget
 
+from ..i18n import N_
+from ..interaction import Interaction
 from ..settings import Settings, mklist
+from ..models import prefs
 from .. import core
-from .. import gitcfg
+from .. import hotkeys
+from .. import icons
 from .. import qtcompat
 from .. import qtutils
 from .. import utils
@@ -22,7 +26,7 @@ class WidgetMixin(object):
     """Mix-in for common utilities and serialization of widget state"""
 
     def __init__(self):
-        self._unmaximized_rect = None
+        self._unmaximized_rect = {}
 
     def center(self):
         parent = self.parent()
@@ -57,101 +61,61 @@ class WidgetMixin(object):
         return self.__class__.__name__.lower()
 
     def save_state(self, settings=None):
-        if gitcfg.current().get('cola.savewindowsettings', True):
+        save = True
+        context = getattr(self, 'context', None)
+        if context:
+            cfg = context.cfg
+            save = cfg.get('cola.savewindowsettings', default=True)
+        if save:
             if settings is None:
                 settings = Settings()
                 settings.load()
             settings.save_gui_state(self)
 
-    def resizeEvent(self, event):
-        super(WidgetMixin, self).resizeEvent(event)
-        # Use a timer to so that the window size and state is up to date.
-        # If we ask for the window state here it will never realize that
-        # we have been maximized because the window state change is processed
-        # after the resize event.  Using a timer event causes it to happen
-        # after all the events have been processsed.
-        # Timer with a delay of zero will trigger immediately after control
-        # returns to the main loop.
-        QtCore.QTimer.singleShot(
-                0, lambda: self._store_unmaximized_dimensions())
-
-    def moveEvent(self, event):
-        super(WidgetMixin, self).moveEvent(event)
-        # as per the QObject::resizeEvent() override
-        QtCore.QTimer.singleShot(
-                0, lambda: self._store_unmaximized_dimensions())
-
-    def _store_unmaximized_dimensions(self):
-        state = self.windowState()
-        maximized = bool(state & Qt.WindowMaximized)
-        if not maximized:
-            size = self.size()
-            width, height = size.width(), size.height()
-            x, y = self.x(), self.y()
-            # XXX can width and height ever not be over zero?
-            if width > 0 and height > 0:
-                self._unmaximized_rect = (x, y, width, height)
-
     def restore_state(self, settings=None):
         if settings is None:
             settings = Settings()
             settings.load()
         state = settings.get_gui_state(self)
-        return bool(state) and self.apply_state(state)
+        if state:
+            result = self.apply_state(state)
+        else:
+            result = False
+        return result
 
     def apply_state(self, state):
         """Imports data for view save/restore"""
-        result = True
-        try:
-            width, height = int(state['width']), int(state['height'])
+
+        width = utils.asint(state.get('width'))
+        height = utils.asint(state.get('height'))
+        x = utils.asint(state.get('x'))
+        y = utils.asint(state.get('y'))
+
+        geometry = state.get('geometry', '')
+        if geometry:
+            from_base64 = QtCore.QByteArray.fromBase64
+            result = self.restoreGeometry(from_base64(core.encode(geometry)))
+        elif width and height:
+            # Users migrating from older versions won't have 'geometry'.
+            # They'll be upgraded to the new format on shutdown.
             self.resize(width, height)
-
-            x, y = int(state['x']), int(state['y'])
             self.move(x, y)
-
-            # calling resize/move won't invoke QWidget::{resize,move}Event
-            # so store the unmaximized size if we properly restored.
-            self._unmaximized_rect = (x, y, width, height)
-        except:
+            result = True
+        else:
             result = False
-        try:
-            if state['maximized']:
-                try:
-                    if utils.is_win32() or utils.is_darwin():
-                        self.resize_to_desktop()
-                    else:
-                        self.showMaximized()
-                except:
-                    pass
-        except:
-            result = False
-        self._apply_state_applied = result
         return result
 
     def export_state(self):
         """Exports data for view save/restore"""
-        state = self.windowState()
-        maximized = bool(state & Qt.WindowMaximized)
-
-        ret = {
-            'maximized': maximized,
-        }
-
-        # when maximized we don't want to overwrite saved x/y/width/height with
-        # desktop dimensions.
-        if maximized:
-            rect = self._unmaximized_rect
-            try:
-                ret['x'], ret['y'], ret['width'], ret['height'] = rect
-            except:
-                pass
-        else:
-            ret['width'] = self.width()
-            ret['height'] = self.height()
-            ret['x'] = self.x()
-            ret['y'] = self.y()
-
-        return ret
+        state = {}
+        geometry = self.saveGeometry()
+        state['geometry'] = geometry.toBase64().data().decode('ascii')
+        # Until 2020: co-exist with older versions
+        state['width'] = self.width()
+        state['height'] = self.height()
+        state['x'] = self.x()
+        state['y'] = self.y()
+        return state
 
     def save_settings(self, settings=None):
         return self.save_state(settings=settings)
@@ -199,9 +163,10 @@ class MainWindowMixin(WidgetMixin):
 
     def save_settings(self, settings=None):
         if settings is None:
+            context = getattr(self, 'context', None)
             settings = Settings()
             settings.load()
-            settings.add_recent(core.getcwd())
+            settings.add_recent(core.getcwd(), prefs.maxrecent(context))
         return WidgetMixin.save_settings(self, settings=settings)
 
     def apply_state(self, state):
@@ -212,11 +177,13 @@ class MainWindowMixin(WidgetMixin):
         else:
             from_base64 = QtCore.QByteArray.fromBase64
             result = self.restoreState(
-                    from_base64(core.encode(windowstate)),
-                    self.widget_version) and result
+                from_base64(core.encode(windowstate)),
+                self.widget_version) and result
+
         self.lock_layout = state.get('lock_layout', self.lock_layout)
         self.update_dockwidget_lock_state()
         self.update_dockwidget_tooltips()
+
         return result
 
     def set_lock_layout(self, lock_layout):
@@ -240,6 +207,45 @@ class MainWindowMixin(WidgetMixin):
             widget.titleBarWidget().update_tooltips()
 
 
+class ListWidget(QtWidgets.QListWidget):
+    """QListWidget with vim j/k navigation hotkeys"""
+
+    def __init__(self, parent=None):
+        super(ListWidget, self).__init__(parent)
+
+        self.up_action = qtutils.add_action(
+            self, N_('Move Up'), self.move_up,
+            hotkeys.MOVE_UP, hotkeys.MOVE_UP_SECONDARY)
+
+        self.down_action = qtutils.add_action(
+            self, N_('Move Down'), self.move_down,
+            hotkeys.MOVE_DOWN, hotkeys.MOVE_DOWN_SECONDARY)
+
+    def selected_item(self):
+        return self.currentItem()
+
+    def selected_items(self):
+        return self.selectedItems()
+
+    def move_up(self):
+        self.move(-1)
+
+    def move_down(self):
+        self.move(1)
+
+    def move(self, direction):
+        item = self.selected_item()
+        if item:
+            row = (self.row(item) + direction) % self.count()
+        elif self.count() > 0:
+            row = (self.count() + direction) % self.count()
+        else:
+            return
+        new_item = self.item(row)
+        if new_item:
+            self.setCurrentItem(new_item)
+
+
 class TreeMixin(object):
 
     def __init__(self, widget, Base):
@@ -326,7 +332,9 @@ class TreeMixin(object):
         elif key == Qt.Key_Left and index.parent().isValid():
 
             # File entries have rowCount() == 0
-            if widget.model().itemFromIndex(index).rowCount() == 0:
+            model = widget.model()
+            if (hasattr(model, 'itemFromIndex')
+                    and model.itemFromIndex(index).rowCount() == 0):
                 widget.setCurrentIndex(index.parent())
 
             # Otherwise, do this for collapsed directories only
@@ -482,6 +490,11 @@ class Dialog(WidgetMixin, QtWidgets.QDialog):
     def __init__(self, parent=None):
         QtWidgets.QDialog.__init__(self, parent)
         WidgetMixin.__init__(self)
+        # Disable the Help button hint on Windows
+        if hasattr(Qt, 'WindowContextHelpButtonHint'):
+            help_hint = Qt.WindowContextHelpButtonHint
+            flags = self.windowFlags() & ~help_hint
+            self.setWindowFlags(flags)
 
     def accept(self):
         self.save_settings()
@@ -597,7 +610,7 @@ class ProgressDialog(QtWidgets.QProgressDialog):
     This dialog ignores the ESC key so that it is not
     prematurely closed.
 
-    An thread is spawned to animate the progress label text.
+    A thread is spawned to animate the progress label text.
 
     """
     def __init__(self, title, label, parent):
@@ -608,7 +621,7 @@ class ProgressDialog(QtWidgets.QProgressDialog):
         self.setRange(0, 0)
         self.setMinimumDuration(0)
         self.setCancelButton(None)
-        self.setFont(qtutils.diff_font())
+        self.setFont(qtutils.default_monospace_font())
         self.thread = ProgressAnimationThread(label, self)
         self.thread.updated.connect(self.refresh, type=Qt.QueuedConnection)
 
@@ -676,12 +689,19 @@ class ProgressAnimationThread(QtCore.QThread):
 
 
 class SpinBox(QtWidgets.QSpinBox):
-    def __init__(self, parent=None):
+
+    def __init__(self, parent=None, value=None,
+                 mini=1, maxi=99999, step=0, prefix='', suffix=''):
         QtWidgets.QSpinBox.__init__(self, parent)
-        self.setMinimum(1)
-        self.setMaximum(99999)
-        self.setPrefix('')
-        self.setSuffix('')
+        self.setPrefix(prefix)
+        self.setSuffix(suffix)
+        self.setWrapping(True)
+        self.setMinimum(mini)
+        self.setMaximum(maxi)
+        if step:
+            self.setSingleStep(step)
+        if value is not None:
+            self.setValue(value)
 
 
 def export_header_columns(widget, state):
@@ -705,3 +725,212 @@ def apply_header_columns(widget, state):
         columns = columns[:-1]
     for idx, size in enumerate(columns):
         header.resizeSection(idx, size)
+
+
+class MessageBox(Dialog):
+    """Improved QMessageBox replacement
+
+    QMessageBox has a lot of usability issues.  It sometimes cannot be
+    resized, and it brings along a lots of annoying properties that we'd have
+    to workaround, so we use a simple custom dialog instead.
+
+    """
+    def __init__(self, parent=None, title='', text='',
+                 info='', details='', logo=None, default=False,
+                 ok_icon=None, ok_text='', cancel_text=None, cancel_icon=None):
+
+        Dialog.__init__(self, parent=parent)
+
+        if parent:
+            self.setWindowModality(Qt.WindowModal)
+        if title:
+            self.setWindowTitle(title)
+
+        self.logo_label = QtWidgets.QLabel()
+        if logo:
+            # Render into a 1-inch wide pixmap
+            pixmap = logo.pixmap(defs.large_icon)
+            self.logo_label.setPixmap(pixmap)
+        else:
+            self.logo_label.hide()
+
+        self.text_label = QtWidgets.QLabel()
+        self.text_label.setText(text)
+
+        self.info_label = QtWidgets.QLabel()
+        if info:
+            self.info_label.setText(info)
+        else:
+            self.info_label.hide()
+
+        ok_icon = icons.mkicon(ok_icon, icons.ok)
+        self.button_ok = qtutils.create_button(text=ok_text, icon=ok_icon)
+
+        self.button_toggle_details = qtutils.create_button(
+            text=N_('Show Details...'))
+
+        self.button_close = qtutils.close_button(
+            text=cancel_text, icon=cancel_icon)
+
+        if ok_text:
+            self.button_ok.setText(ok_text)
+        else:
+            self.button_ok.hide()
+
+        if default:
+            self.button_ok.setDefault(True)
+            self.button_ok.setFocus()
+        else:
+            self.button_close.setDefault(True)
+            self.button_close.setFocus()
+
+        self.details_text = QtWidgets.QPlainTextEdit()
+        self.details_text.setReadOnly(True)
+        self.details_text.hide()
+        if details:
+            self.details_text.setFont(qtutils.default_monospace_font())
+            self.details_text.setPlainText(details)
+        else:
+            self.button_toggle_details.hide()
+
+        self.info_layout = qtutils.vbox(
+            defs.large_margin, defs.button_spacing,
+            self.text_label, self.info_label, qtutils.STRETCH)
+
+        self.top_layout = qtutils.hbox(
+            defs.large_margin, defs.button_spacing,
+            self.logo_label, self.info_layout, qtutils.STRETCH)
+
+        self.buttons_layout = qtutils.hbox(
+            defs.no_margin, defs.button_spacing, qtutils.STRETCH,
+            self.button_toggle_details, self.button_close, self.button_ok)
+
+        self.main_layout = qtutils.vbox(
+            defs.margin, defs.button_spacing,
+            self.top_layout,
+            self.buttons_layout,
+            self.details_text)
+        self.main_layout.setStretchFactor(self.details_text, 2)
+        self.setLayout(self.main_layout)
+
+        qtutils.connect_button(self.button_ok, self.accept)
+        qtutils.connect_button(self.button_close, self.reject)
+        qtutils.connect_button(self.button_toggle_details, self.toggle_details)
+        self.init_state(None, self.set_initial_size)
+
+    def set_initial_size(self):
+        width = defs.dialog_w
+        height = defs.msgbox_h
+        self.resize(width, height)
+
+    def toggle_details(self):
+        if self.details_text.isVisible():
+            text = N_('Show Details...')
+            self.details_text.hide()
+            QtCore.QTimer.singleShot(
+                0, lambda: self.resize(self.width(), defs.msgbox_h))
+        else:
+            text = N_('Hide Details..')
+            self.details_text.show()
+            new_height = defs.msgbox_h * 4
+            if self.height() < new_height:
+                QtCore.QTimer.singleShot(
+                    0, lambda: self.resize(self.width(), new_height))
+
+        self.button_toggle_details.setText(text)
+
+    def keyPressEvent(self, event):
+        """Handle Y/N hotkeys"""
+        key = event.key()
+        if key == Qt.Key_Y:
+            QtCore.QTimer.singleShot(0, self.accept)
+        elif key in (Qt.Key_N, Qt.Key_Q):
+            QtCore.QTimer.singleShot(0, self.reject)
+        return Dialog.keyPressEvent(self, event)
+
+    def run(self):
+        self.show()
+        return self.exec_()
+
+
+def confirm(title, text, informative_text, ok_text,
+            icon=None, default=True,
+            cancel_text=None, cancel_icon=None):
+    """Confirm that an action should take place"""
+    cancel_text = cancel_text or N_('Cancel')
+    logo = icons.from_style(QtWidgets.QStyle.SP_MessageBoxQuestion)
+
+    mbox = MessageBox(
+        parent=qtutils.active_window(), title=title, text=text,
+        info=informative_text, ok_text=ok_text, ok_icon=icon,
+        cancel_text=cancel_text, cancel_icon=cancel_icon,
+        logo=logo, default=default)
+
+    return mbox.run() == mbox.Accepted
+
+
+def critical(title, message=None, details=None):
+    """Show a warning with the provided title and message."""
+    if message is None:
+        message = title
+    logo = icons.from_style(QtWidgets.QStyle.SP_MessageBoxCritical)
+    mbox = MessageBox(
+        parent=qtutils.active_window(), title=title, text=message,
+        details=details, logo=logo)
+    mbox.run()
+
+
+def command_error(title, cmd, status, out, err):
+    """Report an error message about a failed command"""
+    details = Interaction.format_out_err(out, err)
+    message = Interaction.format_command_status(cmd, status)
+    critical(title, message=message, details=details)
+
+
+def information(title, message=None, details=None, informative_text=None):
+    """Show information with the provided title and message."""
+    if message is None:
+        message = title
+    mbox = MessageBox(
+        parent=qtutils.active_window(), title=title, text=message,
+        info=informative_text, details=details, logo=icons.cola())
+    mbox.run()
+
+
+def question(title, text, default=True):
+    """Launches a QMessageBox question with the provided title and message.
+    Passing "default=False" will make "No" the default choice."""
+    parent = qtutils.active_window()
+    logo = icons.from_style(QtWidgets.QStyle.SP_MessageBoxQuestion)
+    msgbox = MessageBox(
+        parent=parent, title=title, text=text, default=default, logo=logo,
+        ok_text=N_('Yes'), cancel_text=N_('No'))
+    return msgbox.run() == msgbox.Accepted
+
+
+def save_as(filename, title):
+    return qtutils.save_as(filename, title=title)
+
+
+def async_command(title, cmd, runtask):
+    parent = qtutils.active_window()
+    task = qtutils.SimpleTask(parent, partial(core.run_command, cmd))
+    task.connect(partial(async_command_result, title, cmd))
+    runtask.start(task)
+
+
+def async_command_result(title, cmd, result):
+    status, out, err = result
+    cmd_string = core.list2cmdline(cmd)
+    Interaction.command(title, cmd_string, status, out, err)
+
+
+def install():
+    """Install the GUI-model interaction hooks"""
+    Interaction.critical = staticmethod(critical)
+    Interaction.confirm = staticmethod(confirm)
+    Interaction.question = staticmethod(question)
+    Interaction.information = staticmethod(information)
+    Interaction.command_error = staticmethod(command_error)
+    Interaction.save_as = staticmethod(save_as)
+    Interaction.async_command = staticmethod(async_command)
