@@ -1,4 +1,5 @@
 """Text widgets"""
+# pylint: disable=unexpected-keyword-arg
 from __future__ import division, absolute_import, unicode_literals
 import math
 
@@ -8,10 +9,11 @@ from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 
+from ..models import prefs
+from ..qtutils import get
 from .. import hotkeys
 from .. import qtutils
 from ..i18n import N_
-from ..models import prefs
 from . import defs
 
 
@@ -23,7 +25,7 @@ class LineEdit(QtWidgets.QLineEdit):
 
     cursor_changed = Signal(int, int)
 
-    def __init__(self, parent=None, row=1, get_value=None):
+    def __init__(self, parent=None, row=1, get_value=None, clear_button=False):
         QtWidgets.QLineEdit.__init__(self, parent)
         self._row = row
         if get_value is None:
@@ -31,6 +33,9 @@ class LineEdit(QtWidgets.QLineEdit):
         self._get_value = get_value
         self.cursor_position = LineEditCursorPosition(self, row)
 
+        if clear_button and hasattr(self, 'setClearButtonEnabled'):
+            self.setClearButtonEnabled(True)
+
     def get(self):
         """Return the raw unicode value from Qt"""
         return self.text()
@@ -68,7 +73,6 @@ class LineEditCursorPosition(object):
         self._widget.setCursorPosition(0)
 
 
-
 class BaseTextEditExtension(QtCore.QObject):
 
     def __init__(self, widget, get_value, readonly):
@@ -88,7 +92,6 @@ class BaseTextEditExtension(QtCore.QObject):
         widget.setMinimumSize(QtCore.QSize(1, 1))
         widget.setWordWrapMode(QtGui.QTextOption.WordWrap)
         widget.setLineWrapMode(widget.NoWrap)
-        widget.setCursorWidth(defs.cursor_width)
         if self._readonly:
             widget.setReadOnly(True)
             widget.setAcceptDrops(False)
@@ -110,18 +113,46 @@ class BaseTextEditExtension(QtCore.QObject):
             blocksig = self.widget.blockSignals(True)
 
         # Save cursor position
-        cursor = self.widget.textCursor()
-        position = cursor.position()
+        offset, selection_text = self.offset_and_selection()
+        old_value = get(self.widget)
+
         # Update text
         self.widget.setPlainText(value)
+
         # Restore cursor
-        cursor = self.widget.textCursor()
-        cursor.setPosition(min(position, cursor.position()))
-        self.widget.setTextCursor(cursor)
+        if selection_text and selection_text in value:
+            # If the old selection exists in the new text then re-select it.
+            idx = value.index(selection_text)
+            cursor = self.widget.textCursor()
+            cursor.setPosition(idx)
+            cursor.setPosition(idx + len(selection_text),
+                               QtGui.QTextCursor.KeepAnchor)
+            self.widget.setTextCursor(cursor)
+
+        elif value == old_value:
+            # Otherwise, if the text is identical and there is no selection
+            # then restore the cursor position.
+            cursor = self.widget.textCursor()
+            cursor.setPosition(offset)
+            self.widget.setTextCursor(cursor)
+        else:
+            # If none of the above applied then restore the cursor position.
+            position = max(0, min(offset, len(value) - 1))
+            cursor = self.widget.textCursor()
+            cursor.setPosition(position)
+            self.widget.setTextCursor(cursor)
+            cursor = self.widget.textCursor()
+            cursor.movePosition(QtGui.QTextCursor.StartOfLine)
+            self.widget.setTextCursor(cursor)
 
         if block:
             self.widget.blockSignals(blocksig)
 
+    def set_cursor_position(self, new_position):
+        cursor = self.widget.textCursor()
+        cursor.setPosition(new_position)
+        self.widget.setTextCursor(cursor)
+
     def tabwidth(self):
         return self._tabwidth
 
@@ -137,16 +168,28 @@ class BaseTextEditExtension(QtCore.QObject):
         cursor = self.widget.textCursor()
         offset = min(cursor.position(), len(contents)-1)
         while (offset >= 1 and
-                contents[offset-1] and
-                contents[offset-1] != '\n'):
+               contents[offset-1] and
+               contents[offset-1] != '\n'):
             offset -= 1
         data = contents[offset:]
         if '\n' in data:
-            line, rest = data.split('\n', 1)
+            line, _ = data.split('\n', 1)
         else:
             line = data
         return line
 
+    def cursor(self):
+        return self.widget.textCursor()
+
+    def has_selection(self):
+        return self.cursor().hasSelection()
+
+    def offset_and_selection(self):
+        cursor = self.cursor()
+        offset = cursor.selectionStart()
+        selection_text = cursor.selection().toPlainText()
+        return offset, selection_text
+
     def mouse_press_event(self, event):
         # Move the text cursor so that the right-click events operate
         # on the current position, not the last left-clicked position.
@@ -154,21 +197,24 @@ class BaseTextEditExtension(QtCore.QObject):
         if event.button() == Qt.RightButton:
             if not widget.textCursor().hasSelection():
                 cursor = widget.cursorForPosition(event.pos())
-                widget.setTextCursor(widget.cursorForPosition(event.pos()))
+                widget.setTextCursor(cursor)
 
     # For extension by sub-classes
 
+    # pylint: disable=no-self-use
     def init(self):
         """Called during init for class-specific settings"""
-        pass
+        return
 
+    # pylint: disable=no-self-use,unused-argument
     def set_textwidth(self, width):
         """Set the text width"""
-        pass
+        return
 
+    # pylint: disable=no-self-use,unused-argument
     def set_linebreak(self, brk):
         """Enable word wrapping"""
-        pass
+        return
 
 
 class PlainTextEditExtension(BaseTextEditExtension):
@@ -206,6 +252,9 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
     def set_value(self, value, block=False):
         self.ext.set_value(value, block=block)
 
+    def has_selection(self):
+        return self.ext.has_selection()
+
     def selected_line(self):
         return self.ext.selected_line()
 
@@ -227,7 +276,7 @@ class PlainTextEdit(QtWidgets.QPlainTextEdit):
         if event.modifiers() & Qt.ControlModifier:
             event.ignore()
             return
-        return super(PlainTextEdit, self).wheelEvent(event)
+        super(PlainTextEdit, self).wheelEvent(event)
 
 
 class TextEditExtension(BaseTextEditExtension):
@@ -247,7 +296,6 @@ class TextEditExtension(BaseTextEditExtension):
         self.widget.setLineWrapColumnOrWidth(width)
 
 
-
 class TextEdit(QtWidgets.QTextEdit):
 
     cursor_changed = Signal(int, int)
@@ -257,6 +305,7 @@ class TextEdit(QtWidgets.QTextEdit):
         QtWidgets.QTextEdit.__init__(self, parent)
         self.ext = TextEditExtension(self, get_value, readonly)
         self.cursor_position = self.ext.cursor_position
+        self.expandtab_enabled = False
 
     def get(self):
         """Return the raw unicode value from Qt"""
@@ -281,6 +330,9 @@ class TextEdit(QtWidgets.QTextEdit):
     def set_linebreak(self, brk):
         self.ext.set_linebreak(brk)
 
+    def set_expandtab(self, value):
+        self.expandtab_enabled = value
+
     def mousePressEvent(self, event):
         self.ext.mouse_press_event(event)
         super(TextEdit, self).mousePressEvent(event)
@@ -290,7 +342,30 @@ class TextEdit(QtWidgets.QTextEdit):
         if event.modifiers() & Qt.ControlModifier:
             event.ignore()
             return
-        return super(TextEdit, self).wheelEvent(event)
+        super(TextEdit, self).wheelEvent(event)
+
+    def should_expandtab(self, event):
+        return event.key() == Qt.Key_Tab and self.expandtab_enabled
+
+    def expandtab(self):
+        tabwidth = max(self.ext.tabwidth(), 1)
+        cursor = self.textCursor()
+        cursor.insertText(' ' * tabwidth)
+
+    def keyPressEvent(self, event):
+        expandtab = self.should_expandtab(event)
+        if expandtab:
+            self.expandtab()
+            event.accept()
+        else:
+            QtWidgets.QTextEdit.keyPressEvent(self, event)
+
+    def keyReleaseEvent(self, event):
+        expandtab = self.should_expandtab(event)
+        if expandtab:
+            event.ignore()
+        else:
+            QtWidgets.QTextEdit.keyReleaseEvent(self, event)
 
 
 class TextEditCursorPosition(object):
@@ -320,25 +395,19 @@ class TextEditCursorPosition(object):
         widget.setTextCursor(cursor)
 
 
-def setup_mono_font(widget):
-    widget.setFont(qtutils.diff_font())
-    widget.set_tabwidth(prefs.tabwidth())
-
-
 class MonoTextEdit(PlainTextEdit):
 
-    def __init__(self, parent=None, readonly=False):
+    def __init__(self, context, parent=None, readonly=False):
         PlainTextEdit.__init__(self, parent=parent, readonly=readonly)
-        setup_mono_font(self)
+        self.setFont(qtutils.diff_font(context))
 
 
 def get_value_hinted(widget):
     text = get_stripped(widget)
-    hint = widget.hint.value()
+    hint = get(widget.hint)
     if text == hint:
         return ''
-    else:
-        return text
+    return text
 
 
 class HintWidget(QtCore.QObject):
@@ -391,8 +460,12 @@ class HintWidget(QtCore.QObject):
             }
         """ % env
 
-        if not modern:
-            widget.installEventFilter(self)
+    def init(self):
+        """Defered initialization"""
+        if self.modern:
+            self.widget().setPlaceholderText(self.value())
+        else:
+            self.widget().installEventFilter(self)
             self.enable(True)
 
     def widget(self):
@@ -446,9 +519,10 @@ class HintWidget(QtCore.QObject):
             style = self._hint_style
         else:
             style = self._default_style
-        self._widget.setStyleSheet(style)
+        QtCore.QTimer.singleShot(
+            0, lambda: self._widget.setStyleSheet(style))
 
-    def eventFilter(self, obj, event):
+    def eventFilter(self, _obj, event):
         """Enable/disable hint-mode when focus changes"""
         etype = event.type()
         if etype == QtCore.QEvent.FocusIn:
@@ -467,19 +541,21 @@ class HintWidget(QtCore.QObject):
     def focus_out(self):
         """Re-enable hint-mode when losing focus"""
         widget = self.widget()
-        if not widget.value():
+        if not get(widget):
             self.enable(True)
 
 
 class HintedPlainTextEdit(PlainTextEdit):
     """A hinted plain text edit"""
 
-    def __init__(self, hint, parent=None, readonly=False):
+    def __init__(self, context, hint, parent=None, readonly=False):
         PlainTextEdit.__init__(self, parent=parent,
                                get_value=get_value_hinted,
                                readonly=readonly)
         self.hint = HintWidget(self, hint)
-        setup_mono_font(self)
+        self.hint.init()
+        self.setFont(qtutils.diff_font(context))
+        self.set_tabwidth(prefs.tabwidth(context))
         # Refresh palettes when text changes
         self.textChanged.connect(self.hint.refresh)
 
@@ -494,13 +570,14 @@ class HintedPlainTextEdit(PlainTextEdit):
 class HintedTextEdit(TextEdit):
     """A hinted text edit"""
 
-    def __init__(self, hint, parent=None, readonly=False):
+    def __init__(self, context, hint, parent=None, readonly=False):
         TextEdit.__init__(self, parent=parent,
                           get_value=get_value_hinted, readonly=readonly)
         self.hint = HintWidget(self, hint)
-        setup_mono_font(self)
+        self.hint.init()
         # Refresh palettes when text changes
         self.textChanged.connect(self.hint.refresh)
+        self.setFont(qtutils.diff_font(context))
 
     def set_value(self, value, block=False):
         """Set the widget text or enable hint mode when empty"""
@@ -594,7 +671,7 @@ class VimMixin(object):
                 # The cursor is at the beginning of the line.
                 # Emit a signal so that the parent can e.g. change focus.
                 widget.leave.emit()
-            elif widget.value()[:position].count('\n') == 0:
+            elif get(widget)[:position].count('\n') == 0:
                 # The cursor is in the middle of the first line of text.
                 # We can't go up ~ jump to the beginning of the line.
                 # Select the text if shift is pressed.
@@ -617,8 +694,9 @@ class VimHintedPlainTextEdit(HintedPlainTextEdit):
     Base = HintedPlainTextEdit
     Mixin = VimMixin
 
-    def __init__(self, hint, parent=None):
-        HintedPlainTextEdit.__init__(self, hint, parent=parent, readonly=True)
+    def __init__(self, context, hint, parent=None):
+        HintedPlainTextEdit.__init__(
+            self, context, hint, parent=parent, readonly=True)
         self._mixin = self.Mixin(self)
 
     def move(self, direction, select=False, n=1):
@@ -640,8 +718,8 @@ class VimTextEdit(MonoTextEdit):
     Base = MonoTextEdit
     Mixin = VimMixin
 
-    def __init__(self, parent=None):
-        MonoTextEdit.__init__(self, parent=None, readonly=True)
+    def __init__(self, context, parent=None, readonly=True):
+        MonoTextEdit.__init__(self, context, parent=None, readonly=readonly)
         self._mixin = self.Mixin(self)
 
     def move(self, direction, select=False, n=1):
@@ -656,26 +734,33 @@ class VimTextEdit(MonoTextEdit):
 
 class HintedLineEdit(LineEdit):
 
-    def __init__(self, hint, parent=None):
+    def __init__(self, context, hint, parent=None):
         LineEdit.__init__(self, parent=parent, get_value=get_value_hinted)
         self.hint = HintWidget(self, hint)
-        self.setFont(qtutils.diff_font())
+        self.hint.init()
+        self.setFont(qtutils.diff_font(context))
         self.textChanged.connect(lambda text: self.hint.refresh())
 
 
-def text_dialog(text, title):
+def text_dialog(context, text, title):
     """Show a wall of text in a dialog"""
     parent = qtutils.active_window()
+
     label = QtWidgets.QLabel(parent)
-    label.setFont(qtutils.diff_font())
+    label.setFont(qtutils.diff_font(context))
     label.setText(text)
-    label.setTextInteractionFlags(Qt.NoTextInteraction)
+    label.setMargin(defs.large_margin)
+    text_flags = Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse
+    label.setTextInteractionFlags(text_flags)
 
     widget = QtWidgets.QDialog(parent)
     widget.setWindowModality(Qt.WindowModal)
     widget.setWindowTitle(title)
 
-    layout = qtutils.hbox(defs.margin, defs.spacing, label)
+    scroll = QtWidgets.QScrollArea()
+    scroll.setWidget(label)
+
+    layout = qtutils.hbox(defs.margin, defs.spacing, scroll)
     widget.setLayout(layout)
 
     qtutils.add_action(widget, N_('Close'), widget.accept,
@@ -687,8 +772,8 @@ def text_dialog(text, title):
 class VimTextBrowser(VimTextEdit):
     """Text viewer with line number annotations"""
 
-    def __init__(self, parent=None, readonly=False):
-        VimTextEdit.__init__(self, parent=parent)
+    def __init__(self, context, parent=None, readonly=True):
+        VimTextEdit.__init__(self, context, parent=parent, readonly=readonly)
         self.numbers = LineNumbers(self)
 
     def resizeEvent(self, event):
@@ -743,8 +828,8 @@ class LineNumbers(TextDecorator):
 
     def width_hint(self):
         document = self.editor.document()
-        digits = int(math.log(max(1, document.blockCount()), 10))
-        return defs.margin + self.fontMetrics().width('0') * (digits + 2)
+        digits = int(math.log(max(1, document.blockCount()), 10)) + 2
+        return defs.large_margin + self.fontMetrics().width('0') * digits
 
     def set_highlighted(self, line_number):
         """Set the line to highlight"""
@@ -754,21 +839,19 @@ class LineNumbers(TextDecorator):
         """Paint the line number"""
         QPalette = QtGui.QPalette
         painter = QtGui.QPainter(self)
-        palette = self.palette()
+        editor = self.editor
+        palette = editor.palette()
 
         painter.fillRect(event.rect(), palette.color(QPalette.Base))
 
-        editor = self.editor
         content_offset = editor.contentOffset()
         block = editor.firstVisibleBlock()
-        current_block_number = max(0, self.editor.textCursor().blockNumber())
         width = self.width()
         event_rect_bottom = event.rect().bottom()
 
         highlight = palette.color(QPalette.Highlight)
-        window = palette.color(QPalette.Window)
+        highlighted_text = palette.color(QPalette.HighlightedText)
         disabled = palette.color(QPalette.Disabled, QPalette.Text)
-        painter.setPen(disabled)
 
         while block.isValid():
             block_geom = editor.blockBoundingGeometry(block)
@@ -777,17 +860,17 @@ class LineNumbers(TextDecorator):
                 break
 
             rect = block_geom.translated(content_offset).toRect()
-            block_number = block.blockNumber();
+            block_number = block.blockNumber()
             if block_number == self.highlight_line:
                 painter.fillRect(rect.x(), rect.y(),
                                  width, rect.height(), highlight)
-            elif block_number == current_block_number:
-                painter.fillRect(rect.x(), rect.y(),
-                                 width, rect.height(), window)
+                painter.setPen(highlighted_text)
+            else:
+                painter.setPen(disabled)
 
             number = '%s' % (block_number + 1)
             painter.drawText(rect.x(), rect.y(),
-                             self.width() - (defs.margin * 2),
+                             self.width() - defs.large_margin,
                              rect.height(),
                              Qt.AlignRight | Qt.AlignVCenter,
                              number)
