@@ -1,8 +1,9 @@
 # TODO pynsist commands call it "installdir", entry points call it "scriptdir"
+import os
 try:
     installdir = scriptdir
 except NameError:
     pass
 pythondir = os.path.join(installdir, 'Python')
-os.environ['PATH'] = (
-    pythondir + os.pathsep + pkgdir + os.pathsep + os.environ.get('PATH', ''))
+path = os.environ.get('PATH', '')
+os.environ['PATH'] = os.pathsep.join([pythondir, pkgdir, path])  # noqa
