@@ -44,7 +44,7 @@ class Replacer(object):
 
     def process(self, text):
 
-        return text.replace( self.from_, self.to )
+        return text.replace(self.from_, self.to)
 
 
 class FileHandler(object):
@@ -61,7 +61,7 @@ class FileHandler(object):
         text = self.opener(self.name, "r").read()
 
         for replacer in self.replacers:
-            text = replacer.process( text )
+            text = replacer.process(text)
 
         self.opener(self.name, "w").write(text)
 
@@ -77,6 +77,7 @@ class Remover(object):
         if self.exists(name):
             self.remove(name)
 
+
 class ForceRename(object):
 
     def __init__(self, renamer, remove):
@@ -89,6 +90,7 @@ class ForceRename(object):
         self.remove(to)
         self.renamer(from_, to)
 
+
 class VerboseRename(object):
 
     def __init__(self, renamer, stream):
@@ -99,9 +101,9 @@ class VerboseRename(object):
     def __call__(self, from_, to):
 
         self.stream.write(
-                "Renaming directory '%s' -> '%s'\n"
-                    % (os.path.basename(from_), os.path.basename(to))
-                )
+            "Renaming directory '%s' -> '%s'\n"
+            % (os.path.basename(from_), os.path.basename(to))
+            )
 
         self.renamer(from_, to)
 
@@ -194,10 +196,12 @@ class NullLayout(object):
     def process(self):
         pass
 
+
 class LayoutFactory(object):
     "Creates a layout object"
 
-    def __init__(self, operations_factory, handler_factory, file_helper, dir_helper, verbose, stream, force):
+    def __init__(self, operations_factory, handler_factory,
+                 file_helper, dir_helper, verbose, stream, force):
 
         self.operations_factory = operations_factory
         self.handler_factory = handler_factory
@@ -216,72 +220,76 @@ class LayoutFactory(object):
         renamer = self.file_helper.move
 
         if self.force:
-            remove = self.operations_factory.create_remover(self.file_helper.exists, self.dir_helper.rmtree)
-            renamer = self.operations_factory.create_force_rename(renamer, remove) 
+            remove = self.operations_factory.create_remover(
+                self.file_helper.exists, self.dir_helper.rmtree)
+            renamer = self.operations_factory.create_force_rename(
+                renamer, remove)
 
         if self.verbose:
-            renamer = self.operations_factory.create_verbose_rename(renamer, self.output_stream) 
+            renamer = self.operations_factory.create_verbose_rename(
+                renamer, self.output_stream)
 
         # Build list of directories to process
         directories = [d for d in contents if self.is_underscore_dir(path, d)]
         underscore_directories = [
-                self.handler_factory.create_dir_handler(d, path, renamer)
-                    for d in directories
-                ]
+            self.handler_factory.create_dir_handler(d, path, renamer)
+            for d in directories
+            ]
 
         if not underscore_directories:
             if self.verbose:
                 self.output_stream.write(
-                        "No top level directories starting with an underscore "
-                        "were found in '%s'\n" % path
-                        )
+                    "No top level directories starting with an underscore "
+                    "were found in '%s'\n" % path
+                    )
             return NullLayout()
 
         # Build list of files that are in those directories
         replacers = []
         for handler in underscore_directories:
-            for directory, dirs, files in self.dir_helper.walk(handler.path()):
+            for directory, _, files in self.dir_helper.walk(handler.path()):
                 for f in files:
                     replacers.append(
-                            self.operations_factory.create_replacer(
-                                handler.relative_path(directory, f),
-                                handler.new_relative_path(directory, f)
-                                )
+                        self.operations_factory.create_replacer(
+                            handler.relative_path(directory, f),
+                            handler.new_relative_path(directory, f)
                             )
+                        )
 
         # Build list of handlers to process all files
         filelist = []
-        for root, dirs, files in self.dir_helper.walk(path):
+        for root, _, files in self.dir_helper.walk(path):
             for f in files:
                 if f.endswith(".html"):
                     filelist.append(
-                            self.handler_factory.create_file_handler(
-                                self.file_helper.path_join(root, f),
-                                replacers,
-                                self.file_helper.open_)
-                            )
+                        self.handler_factory.create_file_handler(
+                            self.file_helper.path_join(root, f),
+                            replacers,
+                            self.file_helper.open_)
+                        )
                 if f.endswith(".js"):
                     filelist.append(
-                            self.handler_factory.create_file_handler(
-                                self.file_helper.path_join(root, f),
-                                [self.operations_factory.create_replacer("'_sources/'", "'sources/'")],
-                                self.file_helper.open_
-                                )
+                        self.handler_factory.create_file_handler(
+                            self.file_helper.path_join(root, f),
+                            [self.operations_factory
+                             .create_replacer("'_sources/'", "'sources/'")],
+                            self.file_helper.open_
                             )
+                        )
 
         return Layout(underscore_directories, filelist)
 
     def is_underscore_dir(self, path, directory):
 
-        return (self.dir_helper.is_dir(self.file_helper.path_join(path, directory))
+        return (
+            self.dir_helper.is_dir(self.file_helper.path_join(path, directory))
             and directory.startswith("_"))
 
 
-
 def sphinx_extension(app, exception):
     "Wrapped up as a Sphinx Extension"
 
-    if not app.builder.name in ("html", "dirhtml"):
+    if app.builder.name not in ("html", "dirhtml"):
         return
 
     if not app.config.sphinx_to_github:
@@ -291,35 +299,38 @@ def sphinx_extension(app, exception):
 
     if exception:
         if app.config.sphinx_to_github_verbose:
-            stdout("Sphinx-to-github: Exception raised in main build, doing nothing.")
+            msg = ("Sphinx-to-github: "
+                   "Exception raised in main build, doing nothing.")
+            stdout(msg)
         return
 
     dir_helper = DirHelper(
-            os.path.isdir,
-            os.listdir,
-            os.walk,
-            shutil.rmtree
-            )
+        os.path.isdir,
+        os.listdir,
+        os.walk,
+        shutil.rmtree
+        )
 
     file_helper = FileSystemHelper(
-            lambda f, mode: codecs.open(f, mode, app.config.sphinx_to_github_encoding),
-            os.path.join,
-            shutil.move,
-            os.path.exists
-            )
+        lambda f, mode:
+        codecs.open(f, mode, app.config.sphinx_to_github_encoding),
+        os.path.join,
+        shutil.move,
+        os.path.exists
+        )
 
     operations_factory = OperationsFactory()
     handler_factory = HandlerFactory()
 
     layout_factory = LayoutFactory(
-            operations_factory,
-            handler_factory,
-            file_helper,
-            dir_helper,
-            app.config.sphinx_to_github_verbose,
-            sys.stdout,
-            force=True
-            )
+        operations_factory,
+        handler_factory,
+        file_helper,
+        dir_helper,
+        app.config.sphinx_to_github_verbose,
+        sys.stdout,
+        force=True
+        )
 
     layout = layout_factory.create_layout(app.outdir)
     layout.process()
@@ -339,47 +350,50 @@ def main(args):
 
     usage = "usage: %prog [options] <html directory>"
     parser = OptionParser(usage=usage)
-    parser.add_option("-v","--verbose", action="store_true",
-            dest="verbose", default=False, help="Provides verbose output")
-    parser.add_option("-e","--encoding", action="store",
-            dest="encoding", default="utf-8", help="Encoding for reading and writing files")
+    parser.add_option(
+        "-v", "--verbose", action="store_true",
+        dest="verbose", default=False, help="Provides verbose output")
+    parser.add_option(
+        "-e", "--encoding", action="store",
+        dest="encoding", default="utf-8",
+        help="Encoding for reading and writing files")
     opts, args = parser.parse_args(args)
 
     try:
         path = args[0]
     except IndexError:
         sys.stderr.write(
-                "Error - Expecting path to html directory:"
-                "sphinx-to-github <path>\n"
-                )
+            "Error - Expecting path to html directory:"
+            "sphinx-to-github <path>\n"
+            )
         return
 
     dir_helper = DirHelper(
-            os.path.isdir,
-            os.listdir,
-            os.walk,
-            shutil.rmtree
-            )
+        os.path.isdir,
+        os.listdir,
+        os.walk,
+        shutil.rmtree
+        )
 
     file_helper = FileSystemHelper(
-            lambda f, mode: codecs.open(f, mode, opts.encoding),
-            os.path.join,
-            shutil.move,
-            os.path.exists
-            )
+        lambda f, mode: codecs.open(f, mode, opts.encoding),
+        os.path.join,
+        shutil.move,
+        os.path.exists
+        )
 
     operations_factory = OperationsFactory()
     handler_factory = HandlerFactory()
 
     layout_factory = LayoutFactory(
-            operations_factory,
-            handler_factory,
-            file_helper,
-            dir_helper,
-            opts.verbose,
-            sys.stdout,
-            force=False
-            )
+        operations_factory,
+        handler_factory,
+        file_helper,
+        dir_helper,
+        opts.verbose,
+        sys.stdout,
+        force=False
+        )
 
     layout = layout_factory.create_layout(path)
     layout.process()
