@@ -2,9 +2,6 @@
 """Tests various operations using the cola.git module
 """
 from __future__ import absolute_import, division, unicode_literals
-
-import os
-import time
 import unittest
 
 try:
@@ -13,7 +10,6 @@ except ImportError:
     from mock import patch
 
 from cola import git
-from cola.compat import WIN32
 from cola.git import STDOUT
 
 
@@ -247,7 +243,7 @@ class GitCommandTest(unittest.TestCase):
     def test_version(self):
         """Test running 'git version'"""
         version = self.git.version()[STDOUT]
-        self.failUnless(version.startswith('git version'))
+        self.assertTrue(version.startswith('git version'))
 
     def test_stdout(self):
         """Test overflowing the stdout buffer"""
