@@ -6,23 +6,21 @@ import unittest
 from cola import core
 from cola.models import main
 
-from test import helper
+from . import helper
 
 
 class MainModelTestCase(helper.GitRepositoryTestCase):
     """Tests the MainModel class."""
 
-    def setUp(self):
-        helper.GitRepositoryTestCase.setUp(self)
-        self.model = main.MainModel(cwd=core.getcwd())
-
     def test_project(self):
         """Test the 'project' attribute."""
         project = os.path.basename(self.test_path())
+        self.model.set_worktree(core.getcwd())
         self.assertEqual(self.model.project, project)
 
     def test_local_branches(self):
         """Test the 'local_branches' attribute."""
+        self.commit_files()
         self.model.update_status()
         self.assertEqual(self.model.local_branches, ['master'])
 
@@ -31,8 +29,9 @@ class MainModelTestCase(helper.GitRepositoryTestCase):
         self.model.update_status()
         self.assertEqual(self.model.remote_branches, [])
 
-        self.git('remote', 'add', 'origin', '.')
-        self.git('fetch', 'origin')
+        self.commit_files()
+        self.run_git('remote', 'add', 'origin', '.')
+        self.run_git('fetch', 'origin')
         self.model.update_status()
         self.assertEqual(self.model.remote_branches, ['origin/master'])
 
@@ -57,19 +56,20 @@ class MainModelTestCase(helper.GitRepositoryTestCase):
 
     def test_remotes(self):
         """Test the 'remote' attribute."""
-        self.git('remote', 'add', 'origin', '.')
+        self.run_git('remote', 'add', 'origin', '.')
         self.model.update_status()
         self.assertEqual(self.model.remotes, ['origin'])
 
     def test_currentbranch(self):
         """Test the 'currentbranch' attribute."""
-        self.git('checkout', '-b', 'test')
+        self.run_git('checkout', '-b', 'test')
         self.model.update_status()
         self.assertEqual(self.model.currentbranch, 'test')
 
     def test_tags(self):
         """Test the 'tags' attribute."""
-        self.git('tag', 'test')
+        self.commit_files()
+        self.run_git('tag', 'test')
         self.model.update_status()
         self.assertEqual(self.model.tags, ['test'])
 
@@ -187,7 +187,7 @@ class RemoteArgsTestCase(unittest.TestCase):
         self.assertFalse('rebase' in kwargs)
 
     def test_remote_args_rebase_only(self):
-        (args, kwargs) = \
+        (_, kwargs) = \
             main.remote_args(self.remote,
                              pull=True,
                              rebase=True,
